# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

IMAGE_MATH_METADATA = Metadata(
    id="fadbb78c295789002a291f08bcdf90f7f7e37437.boutiques",
    name="ImageMath",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_ImageMathParamsDictNoTag = typing.TypedDict('_ImageMathParamsDictNoTag', {
    "image_dimension": typing.Literal[2, 3, 4],
    "output_image": str,
    "operations_and_inputs": str,
    "image1": InputPathType,
    "image2": typing.NotRequired[InputPathType | None],
})
ImageMathParamsDictTagged = typing.TypedDict('ImageMathParamsDictTagged', {
    "@type": typing.Literal["ants/ImageMath"],
    "image_dimension": typing.Literal[2, 3, 4],
    "output_image": str,
    "operations_and_inputs": str,
    "image1": InputPathType,
    "image2": typing.NotRequired[InputPathType | None],
})
ImageMathParamsDict = _ImageMathParamsDictNoTag | ImageMathParamsDictTagged


class ImageMathOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ImageMathParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """The resulting image after processing."""


def image_math_params(
    image_dimension: typing.Literal[2, 3, 4],
    output_image: str,
    operations_and_inputs: str,
    image1: InputPathType,
    image2: InputPathType | None = None,
) -> ImageMathParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the image. Use 2 or 3 for\
            spatial images, and 4 for 4D images like time-series data.
        output_image: The output image file resulting from the operations.
        operations_and_inputs: Mathematical operations and inputs to be applied\
            on the images.
        image1: The first input image for the operation.
        image2: The second input image for the operation, if required.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ImageMath",
        "image_dimension": image_dimension,
        "output_image": output_image,
        "operations_and_inputs": operations_and_inputs,
        "image1": image1,
    }
    if image2 is not None:
        params["image2"] = image2
    return params


def image_math_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ImageMathParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[2, 3, 4]`')
    if params["image_dimension"] not in [2, 3, 4]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [2, 3, 4]")
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("operations_and_inputs", None) is None:
        raise StyxValidationError("`operations_and_inputs` must not be None")
    if not isinstance(params["operations_and_inputs"], str):
        raise StyxValidationError(f'`operations_and_inputs` has the wrong type: Received `{type(params.get("operations_and_inputs", None))}` expected `str`')
    if params.get("image1", None) is None:
        raise StyxValidationError("`image1` must not be None")
    if not isinstance(params["image1"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image1` has the wrong type: Received `{type(params.get("image1", None))}` expected `InputPathType`')
    if params.get("image2", None) is not None:
        if not isinstance(params["image2"], (pathlib.Path, str)):
            raise StyxValidationError(f'`image2` has the wrong type: Received `{type(params.get("image2", None))}` expected `InputPathType | None`')


def image_math_cargs(
    params: ImageMathParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ImageMath")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(params.get("output_image", None))
    cargs.append(params.get("operations_and_inputs", None))
    cargs.append(execution.input_file(params.get("image1", None)))
    if params.get("image2", None) is not None:
        cargs.append(execution.input_file(params.get("image2", None)))
    return cargs


def image_math_outputs(
    params: ImageMathParamsDict,
    execution: Execution,
) -> ImageMathOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ImageMathOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file(params.get("output_image", None)),
    )
    return ret


def image_math_execute(
    params: ImageMathParamsDict,
    runner: Runner | None = None,
) -> ImageMathOutputs:
    """
    ImageMath
    
    A versatile tool for performing various mathematical and manipulation
    operations on images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImageMathOutputs`).
    """
    image_math_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMAGE_MATH_METADATA)
    params = execution.params(params)
    cargs = image_math_cargs(params, execution)
    ret = image_math_outputs(params, execution)
    execution.run(cargs)
    return ret


def image_math(
    image_dimension: typing.Literal[2, 3, 4],
    output_image: str,
    operations_and_inputs: str,
    image1: InputPathType,
    image2: InputPathType | None = None,
    runner: Runner | None = None,
) -> ImageMathOutputs:
    """
    ImageMath
    
    A versatile tool for performing various mathematical and manipulation
    operations on images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the image. Use 2 or 3 for\
            spatial images, and 4 for 4D images like time-series data.
        output_image: The output image file resulting from the operations.
        operations_and_inputs: Mathematical operations and inputs to be applied\
            on the images.
        image1: The first input image for the operation.
        image2: The second input image for the operation, if required.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImageMathOutputs`).
    """
    params = image_math_params(
        image_dimension=image_dimension,
        output_image=output_image,
        operations_and_inputs=operations_and_inputs,
        image1=image1,
        image2=image2,
    )
    return image_math_execute(params, runner)


__all__ = [
    "IMAGE_MATH_METADATA",
    "ImageMathOutputs",
    "ImageMathParamsDict",
    "ImageMathParamsDictTagged",
    "image_math",
    "image_math_execute",
    "image_math_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_LANDMARK_BASED_TRANSFORM_INITIALIZER_METADATA = Metadata(
    id="e3b5bf9f02ef1ef7f7316f607332292a3b1568a6.boutiques",
    name="antsLandmarkBasedTransformInitializer",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsLandmarkBasedTransformInitializerParamsDictNoTag = typing.TypedDict('_AntsLandmarkBasedTransformInitializerParamsDictNoTag', {
    "dimension": int,
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "transform_type": typing.Literal["rigid", "affine", "bspline"],
    "output_transform": str,
    "mesh_size": typing.NotRequired[str | None],
    "number_of_levels": typing.NotRequired[int | None],
    "order": typing.NotRequired[int | None],
    "enforce_stationary_boundaries": typing.NotRequired[bool | None],
    "landmark_weights": typing.NotRequired[InputPathType | None],
})
AntsLandmarkBasedTransformInitializerParamsDictTagged = typing.TypedDict('AntsLandmarkBasedTransformInitializerParamsDictTagged', {
    "@type": typing.Literal["ants/antsLandmarkBasedTransformInitializer"],
    "dimension": int,
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "transform_type": typing.Literal["rigid", "affine", "bspline"],
    "output_transform": str,
    "mesh_size": typing.NotRequired[str | None],
    "number_of_levels": typing.NotRequired[int | None],
    "order": typing.NotRequired[int | None],
    "enforce_stationary_boundaries": typing.NotRequired[bool | None],
    "landmark_weights": typing.NotRequired[InputPathType | None],
})
AntsLandmarkBasedTransformInitializerParamsDict = _AntsLandmarkBasedTransformInitializerParamsDictNoTag | AntsLandmarkBasedTransformInitializerParamsDictTagged


class AntsLandmarkBasedTransformInitializerOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsLandmarkBasedTransformInitializerParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform: OutputPathType
    """The output file containing the initialized transform."""


def ants_landmark_based_transform_initializer_params(
    dimension: int,
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine", "bspline"],
    output_transform: str,
    mesh_size: str | None = None,
    number_of_levels: int | None = None,
    order: int | None = None,
    enforce_stationary_boundaries: bool | None = None,
    landmark_weights: InputPathType | None = None,
) -> AntsLandmarkBasedTransformInitializerParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimension: The dimensionality of the registration problem (e.g., 2 for\
            2D, 3 for 3D).
        fixed_image: The fixed image in the registration process.
        moving_image: The moving image in the registration process.
        transform_type: The type of transform to initialize. Options are\
            'rigid', 'affine', or 'bspline'.
        output_transform: The output transform file that will be created.
        mesh_size: The mesh size for the B-spline transform, specified as\
            'meshSize[0]xmeshSize[1]x...'. Default is '1x1x1'.
        number_of_levels: Number of levels for multi-resolution fitting.\
            Default is 4.
        order: The polynomial order of the B-spline transform. Default is 3.
        enforce_stationary_boundaries: Enforces stationary boundaries for the\
            B-spline transform. Default is 1 (true).
        landmark_weights: File containing landmark weights. Each row is either\
            'label,labelWeight' or 'labelWeight'.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsLandmarkBasedTransformInitializer",
        "dimension": dimension,
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "transform_type": transform_type,
        "output_transform": output_transform,
    }
    if mesh_size is not None:
        params["mesh_size"] = mesh_size
    if number_of_levels is not None:
        params["number_of_levels"] = number_of_levels
    if order is not None:
        params["order"] = order
    if enforce_stationary_boundaries is not None:
        params["enforce_stationary_boundaries"] = enforce_stationary_boundaries
    if landmark_weights is not None:
        params["landmark_weights"] = landmark_weights
    return params


def ants_landmark_based_transform_initializer_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsLandmarkBasedTransformInitializerParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimension", None) is None:
        raise StyxValidationError("`dimension` must not be None")
    if not isinstance(params["dimension"], int):
        raise StyxValidationError(f'`dimension` has the wrong type: Received `{type(params.get("dimension", None))}` expected `int`')
    if params.get("fixed_image", None) is None:
        raise StyxValidationError("`fixed_image` must not be None")
    if not isinstance(params["fixed_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixed_image` has the wrong type: Received `{type(params.get("fixed_image", None))}` expected `InputPathType`')
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("transform_type", None) is None:
        raise StyxValidationError("`transform_type` must not be None")
    if not isinstance(params["transform_type"], str):
        raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal["rigid", "affine", "bspline"]`')
    if params["transform_type"] not in ["rigid", "affine", "bspline"]:
        raise StyxValidationError("Parameter `transform_type` must be one of [\"rigid\", \"affine\", \"bspline\"]")
    if params.get("output_transform", None) is None:
        raise StyxValidationError("`output_transform` must not be None")
    if not isinstance(params["output_transform"], str):
        raise StyxValidationError(f'`output_transform` has the wrong type: Received `{type(params.get("output_transform", None))}` expected `str`')
    if params.get("mesh_size", None) is not None:
        if not isinstance(params["mesh_size"], str):
            raise StyxValidationError(f'`mesh_size` has the wrong type: Received `{type(params.get("mesh_size", None))}` expected `str | None`')
    if params.get("number_of_levels", None) is not None:
        if not isinstance(params["number_of_levels"], int):
            raise StyxValidationError(f'`number_of_levels` has the wrong type: Received `{type(params.get("number_of_levels", None))}` expected `int | None`')
    if params.get("order", None) is not None:
        if not isinstance(params["order"], int):
            raise StyxValidationError(f'`order` has the wrong type: Received `{type(params.get("order", None))}` expected `int | None`')
    if params.get("enforce_stationary_boundaries", None) is not None:
        if not isinstance(params["enforce_stationary_boundaries"], bool):
            raise StyxValidationError(f'`enforce_stationary_boundaries` has the wrong type: Received `{type(params.get("enforce_stationary_boundaries", None))}` expected `bool | None`')
    if params.get("landmark_weights", None) is not None:
        if not isinstance(params["landmark_weights"], (pathlib.Path, str)):
            raise StyxValidationError(f'`landmark_weights` has the wrong type: Received `{type(params.get("landmark_weights", None))}` expected `InputPathType | None`')


def ants_landmark_based_transform_initializer_cargs(
    params: AntsLandmarkBasedTransformInitializerParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsLandmarkBasedTransformInitializer")
    cargs.append(str(params.get("dimension", None)))
    cargs.append(execution.input_file(params.get("fixed_image", None)))
    cargs.append(execution.input_file(params.get("moving_image", None)))
    cargs.append(params.get("transform_type", None))
    cargs.append(params.get("output_transform", None))
    if params.get("mesh_size", None) is not None:
        cargs.append(params.get("mesh_size", None))
    if params.get("number_of_levels", None) is not None:
        cargs.append(str(params.get("number_of_levels", None)))
    if params.get("order", None) is not None:
        cargs.append(str(params.get("order", None)))
    if params.get("enforce_stationary_boundaries", None) is not None:
        cargs.append(("1" if params.get("enforce_stationary_boundaries", None) else "0"))
    if params.get("landmark_weights", None) is not None:
        cargs.append(execution.input_file(params.get("landmark_weights", None)))
    return cargs


def ants_landmark_based_transform_initializer_outputs(
    params: AntsLandmarkBasedTransformInitializerParamsDict,
    execution: Execution,
) -> AntsLandmarkBasedTransformInitializerOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsLandmarkBasedTransformInitializerOutputs(
        root=execution.output_file("."),
        output_transform=execution.output_file(params.get("output_transform", None)),
    )
    return ret


def ants_landmark_based_transform_initializer_execute(
    params: AntsLandmarkBasedTransformInitializerParamsDict,
    runner: Runner | None = None,
) -> AntsLandmarkBasedTransformInitializerOutputs:
    """
    antsLandmarkBasedTransformInitializer
    
    This tool initializes a transform between two images based on corresponding
    landmarks.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsLandmarkBasedTransformInitializerOutputs`).
    """
    ants_landmark_based_transform_initializer_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_LANDMARK_BASED_TRANSFORM_INITIALIZER_METADATA)
    params = execution.params(params)
    cargs = ants_landmark_based_transform_initializer_cargs(params, execution)
    ret = ants_landmark_based_transform_initializer_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_landmark_based_transform_initializer(
    dimension: int,
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine", "bspline"],
    output_transform: str,
    mesh_size: str | None = None,
    number_of_levels: int | None = None,
    order: int | None = None,
    enforce_stationary_boundaries: bool | None = None,
    landmark_weights: InputPathType | None = None,
    runner: Runner | None = None,
) -> AntsLandmarkBasedTransformInitializerOutputs:
    """
    antsLandmarkBasedTransformInitializer
    
    This tool initializes a transform between two images based on corresponding
    landmarks.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: The dimensionality of the registration problem (e.g., 2 for\
            2D, 3 for 3D).
        fixed_image: The fixed image in the registration process.
        moving_image: The moving image in the registration process.
        transform_type: The type of transform to initialize. Options are\
            'rigid', 'affine', or 'bspline'.
        output_transform: The output transform file that will be created.
        mesh_size: The mesh size for the B-spline transform, specified as\
            'meshSize[0]xmeshSize[1]x...'. Default is '1x1x1'.
        number_of_levels: Number of levels for multi-resolution fitting.\
            Default is 4.
        order: The polynomial order of the B-spline transform. Default is 3.
        enforce_stationary_boundaries: Enforces stationary boundaries for the\
            B-spline transform. Default is 1 (true).
        landmark_weights: File containing landmark weights. Each row is either\
            'label,labelWeight' or 'labelWeight'.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsLandmarkBasedTransformInitializerOutputs`).
    """
    params = ants_landmark_based_transform_initializer_params(
        dimension=dimension,
        fixed_image=fixed_image,
        moving_image=moving_image,
        transform_type=transform_type,
        output_transform=output_transform,
        mesh_size=mesh_size,
        number_of_levels=number_of_levels,
        order=order,
        enforce_stationary_boundaries=enforce_stationary_boundaries,
        landmark_weights=landmark_weights,
    )
    return ants_landmark_based_transform_initializer_execute(params, runner)


__all__ = [
    "ANTS_LANDMARK_BASED_TRANSFORM_INITIALIZER_METADATA",
    "AntsLandmarkBasedTransformInitializerOutputs",
    "AntsLandmarkBasedTransformInitializerParamsDict",
    "AntsLandmarkBasedTransformInitializerParamsDictTagged",
    "ants_landmark_based_transform_initializer",
    "ants_landmark_based_transform_initializer_execute",
    "ants_landmark_based_transform_initializer_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_BASED_SMOOTHING_METADATA = Metadata(
    id="b2f8fe5e700adfbe15932f192ed3a9cf1462c8c5.boutiques",
    name="SurfaceBasedSmoothing",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_SurfaceBasedSmoothingParamsDictNoTag = typing.TypedDict('_SurfaceBasedSmoothingParamsDictNoTag', {
    "image_to_smooth": InputPathType,
    "sigma": float,
    "surface_image": InputPathType,
    "outname": str,
    "num_repeats": typing.NotRequired[int | None],
})
SurfaceBasedSmoothingParamsDictTagged = typing.TypedDict('SurfaceBasedSmoothingParamsDictTagged', {
    "@type": typing.Literal["ants/SurfaceBasedSmoothing"],
    "image_to_smooth": InputPathType,
    "sigma": float,
    "surface_image": InputPathType,
    "outname": str,
    "num_repeats": typing.NotRequired[int | None],
})
SurfaceBasedSmoothingParamsDict = _SurfaceBasedSmoothingParamsDictNoTag | SurfaceBasedSmoothingParamsDictTagged


class SurfaceBasedSmoothingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceBasedSmoothingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    smoothed_output: OutputPathType
    """The output smoothed image."""


def surface_based_smoothing_params(
    image_to_smooth: InputPathType,
    sigma: float,
    surface_image: InputPathType,
    outname: str,
    num_repeats: int | None = None,
) -> SurfaceBasedSmoothingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_to_smooth: The image that needs to be smoothed.
        sigma: Geodesic neighborhood radius.
        surface_image: Assumes a label == 1 that defines the surface.
        outname: The name of the output file.
        num_repeats: Number of times the geodesic neighborhood is applied\
            repeatedly.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/SurfaceBasedSmoothing",
        "image_to_smooth": image_to_smooth,
        "sigma": sigma,
        "surface_image": surface_image,
        "outname": outname,
    }
    if num_repeats is not None:
        params["num_repeats"] = num_repeats
    return params


def surface_based_smoothing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceBasedSmoothingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_to_smooth", None) is None:
        raise StyxValidationError("`image_to_smooth` must not be None")
    if not isinstance(params["image_to_smooth"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image_to_smooth` has the wrong type: Received `{type(params.get("image_to_smooth", None))}` expected `InputPathType`')
    if params.get("sigma", None) is None:
        raise StyxValidationError("`sigma` must not be None")
    if not isinstance(params["sigma"], (float, int)):
        raise StyxValidationError(f'`sigma` has the wrong type: Received `{type(params.get("sigma", None))}` expected `float`')
    if params.get("surface_image", None) is None:
        raise StyxValidationError("`surface_image` must not be None")
    if not isinstance(params["surface_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface_image` has the wrong type: Received `{type(params.get("surface_image", None))}` expected `InputPathType`')
    if params.get("outname", None) is None:
        raise StyxValidationError("`outname` must not be None")
    if not isinstance(params["outname"], str):
        raise StyxValidationError(f'`outname` has the wrong type: Received `{type(params.get("outname", None))}` expected `str`')
    if params.get("num_repeats", None) is not None:
        if not isinstance(params["num_repeats"], int):
            raise StyxValidationError(f'`num_repeats` has the wrong type: Received `{type(params.get("num_repeats", None))}` expected `int | None`')


def surface_based_smoothing_cargs(
    params: SurfaceBasedSmoothingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("SurfaceBasedSmoothing")
    cargs.append(execution.input_file(params.get("image_to_smooth", None)))
    cargs.append(str(params.get("sigma", None)))
    cargs.append(execution.input_file(params.get("surface_image", None)))
    cargs.append(params.get("outname", None))
    if params.get("num_repeats", None) is not None:
        cargs.append(str(params.get("num_repeats", None)))
    return cargs


def surface_based_smoothing_outputs(
    params: SurfaceBasedSmoothingParamsDict,
    execution: Execution,
) -> SurfaceBasedSmoothingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceBasedSmoothingOutputs(
        root=execution.output_file("."),
        smoothed_output=execution.output_file(params.get("outname", None)),
    )
    return ret


def surface_based_smoothing_execute(
    params: SurfaceBasedSmoothingParamsDict,
    runner: Runner | None = None,
) -> SurfaceBasedSmoothingOutputs:
    """
    SurfaceBasedSmoothing
    
    Surface-based smoothing applied to ImageToSmooth using a geodesic
    neighbourhood defined by sigma and the surface image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceBasedSmoothingOutputs`).
    """
    surface_based_smoothing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_BASED_SMOOTHING_METADATA)
    params = execution.params(params)
    cargs = surface_based_smoothing_cargs(params, execution)
    ret = surface_based_smoothing_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_based_smoothing(
    image_to_smooth: InputPathType,
    sigma: float,
    surface_image: InputPathType,
    outname: str,
    num_repeats: int | None = None,
    runner: Runner | None = None,
) -> SurfaceBasedSmoothingOutputs:
    """
    SurfaceBasedSmoothing
    
    Surface-based smoothing applied to ImageToSmooth using a geodesic
    neighbourhood defined by sigma and the surface image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_to_smooth: The image that needs to be smoothed.
        sigma: Geodesic neighborhood radius.
        surface_image: Assumes a label == 1 that defines the surface.
        outname: The name of the output file.
        num_repeats: Number of times the geodesic neighborhood is applied\
            repeatedly.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceBasedSmoothingOutputs`).
    """
    params = surface_based_smoothing_params(
        image_to_smooth=image_to_smooth,
        sigma=sigma,
        surface_image=surface_image,
        outname=outname,
        num_repeats=num_repeats,
    )
    return surface_based_smoothing_execute(params, runner)


__all__ = [
    "SURFACE_BASED_SMOOTHING_METADATA",
    "SurfaceBasedSmoothingOutputs",
    "SurfaceBasedSmoothingParamsDict",
    "SurfaceBasedSmoothingParamsDictTagged",
    "surface_based_smoothing",
    "surface_based_smoothing_execute",
    "surface_based_smoothing_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SET_SPACING_METADATA = Metadata(
    id="943a10a041bed9edc5b49fa9bf6fd81849eff416.boutiques",
    name="SetSpacing",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_SetSpacingParamsDictNoTag = typing.TypedDict('_SetSpacingParamsDictNoTag', {
    "dimension": int,
    "input_file": InputPathType,
    "output_file": str,
    "spacing": list[float],
})
SetSpacingParamsDictTagged = typing.TypedDict('SetSpacingParamsDictTagged', {
    "@type": typing.Literal["ants/SetSpacing"],
    "dimension": int,
    "input_file": InputPathType,
    "output_file": str,
    "spacing": list[float],
})
SetSpacingParamsDict = _SetSpacingParamsDictNoTag | SetSpacingParamsDictTagged


class SetSpacingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SetSpacingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """The output image with the specified spacing."""


def set_spacing_params(
    dimension: int,
    input_file: InputPathType,
    output_file: str,
    spacing: list[float],
) -> SetSpacingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimension: The dimensionality of the image (e.g., 2 or 3).
        input_file: The input image file in HDR format.
        output_file: The output image file in NII format.
        spacing: Spacing values for each dimension. Requires SpacingX,\
            SpacingY, and optionally SpacingZ.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/SetSpacing",
        "dimension": dimension,
        "input_file": input_file,
        "output_file": output_file,
        "spacing": spacing,
    }
    return params


def set_spacing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SetSpacingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimension", None) is None:
        raise StyxValidationError("`dimension` must not be None")
    if not isinstance(params["dimension"], int):
        raise StyxValidationError(f'`dimension` has the wrong type: Received `{type(params.get("dimension", None))}` expected `int`')
    if params.get("input_file", None) is None:
        raise StyxValidationError("`input_file` must not be None")
    if not isinstance(params["input_file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_file` has the wrong type: Received `{type(params.get("input_file", None))}` expected `InputPathType`')
    if params.get("output_file", None) is None:
        raise StyxValidationError("`output_file` must not be None")
    if not isinstance(params["output_file"], str):
        raise StyxValidationError(f'`output_file` has the wrong type: Received `{type(params.get("output_file", None))}` expected `str`')
    if params.get("spacing", None) is None:
        raise StyxValidationError("`spacing` must not be None")
    if not isinstance(params["spacing"], list):
        raise StyxValidationError(f'`spacing` has the wrong type: Received `{type(params.get("spacing", None))}` expected `list[float]`')
    if not (2 <= len(params["spacing"]) <= 3):
        raise StyxValidationError("Parameter `spacing` must contain between 2 and 3 elements (inclusive)")
    for e in params["spacing"]:
        if not isinstance(e, (float, int)):
            raise StyxValidationError(f'`spacing` has the wrong type: Received `{type(params.get("spacing", None))}` expected `list[float]`')


def set_spacing_cargs(
    params: SetSpacingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("SetSpacing")
    cargs.append(str(params.get("dimension", None)))
    cargs.append(execution.input_file(params.get("input_file", None)))
    cargs.append(params.get("output_file", None))
    cargs.extend(map(str, params.get("spacing", None)))
    return cargs


def set_spacing_outputs(
    params: SetSpacingParamsDict,
    execution: Execution,
) -> SetSpacingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SetSpacingOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file(params.get("output_file", None)),
    )
    return ret


def set_spacing_execute(
    params: SetSpacingParamsDict,
    runner: Runner | None = None,
) -> SetSpacingOutputs:
    """
    SetSpacing
    
    A tool to set the spacing of an image in each dimension.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetSpacingOutputs`).
    """
    set_spacing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SET_SPACING_METADATA)
    params = execution.params(params)
    cargs = set_spacing_cargs(params, execution)
    ret = set_spacing_outputs(params, execution)
    execution.run(cargs)
    return ret


def set_spacing(
    dimension: int,
    input_file: InputPathType,
    output_file: str,
    spacing: list[float],
    runner: Runner | None = None,
) -> SetSpacingOutputs:
    """
    SetSpacing
    
    A tool to set the spacing of an image in each dimension.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: The dimensionality of the image (e.g., 2 or 3).
        input_file: The input image file in HDR format.
        output_file: The output image file in NII format.
        spacing: Spacing values for each dimension. Requires SpacingX,\
            SpacingY, and optionally SpacingZ.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetSpacingOutputs`).
    """
    params = set_spacing_params(
        dimension=dimension,
        input_file=input_file,
        output_file=output_file,
        spacing=spacing,
    )
    return set_spacing_execute(params, runner)


__all__ = [
    "SET_SPACING_METADATA",
    "SetSpacingOutputs",
    "SetSpacingParamsDict",
    "SetSpacingParamsDictTagged",
    "set_spacing",
    "set_spacing_execute",
    "set_spacing_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SIMULATE_DISPLACEMENT_FIELD_METADATA = Metadata(
    id="62eab1210d3ee9c94268b73a3a247d08a7a705f3.boutiques",
    name="SimulateDisplacementField",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_SimulateDisplacementFieldBsplineOptionsParamsDictNoTag = typing.TypedDict('_SimulateDisplacementFieldBsplineOptionsParamsDictNoTag', {
    "number_of_fitting_levels": typing.NotRequired[int | None],
    "number_of_control_points": typing.NotRequired[int | None],
})
SimulateDisplacementFieldBsplineOptionsParamsDictTagged = typing.TypedDict('SimulateDisplacementFieldBsplineOptionsParamsDictTagged', {
    "@type": typing.Literal["bspline_options"],
    "number_of_fitting_levels": typing.NotRequired[int | None],
    "number_of_control_points": typing.NotRequired[int | None],
})
SimulateDisplacementFieldBsplineOptionsParamsDict = _SimulateDisplacementFieldBsplineOptionsParamsDictNoTag | SimulateDisplacementFieldBsplineOptionsParamsDictTagged


_SimulateDisplacementFieldExponentialOptionsParamsDictNoTag = typing.TypedDict('_SimulateDisplacementFieldExponentialOptionsParamsDictNoTag', {
    "smoothing_standard_deviation": typing.NotRequired[float | None],
})
SimulateDisplacementFieldExponentialOptionsParamsDictTagged = typing.TypedDict('SimulateDisplacementFieldExponentialOptionsParamsDictTagged', {
    "@type": typing.Literal["exponential_options"],
    "smoothing_standard_deviation": typing.NotRequired[float | None],
})
SimulateDisplacementFieldExponentialOptionsParamsDict = _SimulateDisplacementFieldExponentialOptionsParamsDictNoTag | SimulateDisplacementFieldExponentialOptionsParamsDictTagged


_SimulateDisplacementFieldParamsDictNoTag = typing.TypedDict('_SimulateDisplacementFieldParamsDictNoTag', {
    "image_dimension": int,
    "displacement_field_type": typing.Literal["BSpline", "Exponential"],
    "domain_image": InputPathType,
    "output_field": str,
    "number_of_random_points": typing.NotRequired[int | None],
    "standard_deviation_displacement_field": typing.NotRequired[float | None],
    "enforce_stationary_boundary": typing.NotRequired[int | None],
    "displacement_specific_options": typing.NotRequired[typing.Union[SimulateDisplacementFieldBsplineOptionsParamsDictTagged, SimulateDisplacementFieldExponentialOptionsParamsDictTagged] | None],
})
SimulateDisplacementFieldParamsDictTagged = typing.TypedDict('SimulateDisplacementFieldParamsDictTagged', {
    "@type": typing.Literal["ants/SimulateDisplacementField"],
    "image_dimension": int,
    "displacement_field_type": typing.Literal["BSpline", "Exponential"],
    "domain_image": InputPathType,
    "output_field": str,
    "number_of_random_points": typing.NotRequired[int | None],
    "standard_deviation_displacement_field": typing.NotRequired[float | None],
    "enforce_stationary_boundary": typing.NotRequired[int | None],
    "displacement_specific_options": typing.NotRequired[typing.Union[SimulateDisplacementFieldBsplineOptionsParamsDictTagged, SimulateDisplacementFieldExponentialOptionsParamsDictTagged] | None],
})
SimulateDisplacementFieldParamsDict = _SimulateDisplacementFieldParamsDictNoTag | SimulateDisplacementFieldParamsDictTagged


def simulate_displacement_field_displacement_specific_options_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "bspline_options": simulate_displacement_field_bspline_options_cargs,
        "exponential_options": simulate_displacement_field_exponential_options_cargs,
    }.get(t)


def simulate_displacement_field_displacement_specific_options_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "bspline_options": simulate_displacement_field_bspline_options_validate,
        "exponential_options": simulate_displacement_field_exponential_options_validate,
    }.get(t)


def simulate_displacement_field_bspline_options(
    number_of_fitting_levels: int | None = None,
    number_of_control_points: int | None = None,
) -> SimulateDisplacementFieldBsplineOptionsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        number_of_fitting_levels: Number of fitting levels for BSpline.
        number_of_control_points: Number of control points for BSpline.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "bspline_options",
    }
    if number_of_fitting_levels is not None:
        params["number_of_fitting_levels"] = number_of_fitting_levels
    if number_of_control_points is not None:
        params["number_of_control_points"] = number_of_control_points
    return params


def simulate_displacement_field_bspline_options_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SimulateDisplacementFieldBsplineOptionsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("number_of_fitting_levels", None) is not None:
        if not isinstance(params["number_of_fitting_levels"], int):
            raise StyxValidationError(f'`number_of_fitting_levels` has the wrong type: Received `{type(params.get("number_of_fitting_levels", None))}` expected `int | None`')
    if params.get("number_of_control_points", None) is not None:
        if not isinstance(params["number_of_control_points"], int):
            raise StyxValidationError(f'`number_of_control_points` has the wrong type: Received `{type(params.get("number_of_control_points", None))}` expected `int | None`')


def simulate_displacement_field_bspline_options_cargs(
    params: SimulateDisplacementFieldBsplineOptionsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("number_of_fitting_levels", None) is not None:
        cargs.append(str(params.get("number_of_fitting_levels", None)))
    if params.get("number_of_control_points", None) is not None:
        cargs.append(str(params.get("number_of_control_points", None)))
    return cargs


def simulate_displacement_field_exponential_options(
    smoothing_standard_deviation: float | None = None,
) -> SimulateDisplacementFieldExponentialOptionsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        smoothing_standard_deviation: Smoothing standard deviation for\
            Exponential.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exponential_options",
    }
    if smoothing_standard_deviation is not None:
        params["smoothing_standard_deviation"] = smoothing_standard_deviation
    return params


def simulate_displacement_field_exponential_options_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SimulateDisplacementFieldExponentialOptionsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("smoothing_standard_deviation", None) is not None:
        if not isinstance(params["smoothing_standard_deviation"], (float, int)):
            raise StyxValidationError(f'`smoothing_standard_deviation` has the wrong type: Received `{type(params.get("smoothing_standard_deviation", None))}` expected `float | None`')


def simulate_displacement_field_exponential_options_cargs(
    params: SimulateDisplacementFieldExponentialOptionsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("smoothing_standard_deviation", None) is not None:
        cargs.append(str(params.get("smoothing_standard_deviation", None)))
    return cargs


class SimulateDisplacementFieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SimulateDisplacementFieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_displacement_field: OutputPathType
    """The simulated displacement field."""


def simulate_displacement_field_params(
    image_dimension: int,
    displacement_field_type: typing.Literal["BSpline", "Exponential"],
    domain_image: InputPathType,
    output_field: str,
    number_of_random_points: int | None = None,
    standard_deviation_displacement_field: float | None = None,
    enforce_stationary_boundary: int | None = None,
    displacement_specific_options: typing.Union[SimulateDisplacementFieldBsplineOptionsParamsDictTagged, SimulateDisplacementFieldExponentialOptionsParamsDictTagged] | None = None,
) -> SimulateDisplacementFieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Dimensionality of the image.
        displacement_field_type: Type of displacement field to simulate.
        domain_image: Image defining the domain for the displacement field.
        output_field: Path to save the output displacement field.
        number_of_random_points: Number of random points to use in the\
            simulation.
        standard_deviation_displacement_field: Standard deviation for the\
            displacement field.
        enforce_stationary_boundary: Boolean flag indicating whether to enforce\
            stationary boundary.
        displacement_specific_options: Options specific to the type of\
            displacement field simulation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/SimulateDisplacementField",
        "image_dimension": image_dimension,
        "displacement_field_type": displacement_field_type,
        "domain_image": domain_image,
        "output_field": output_field,
    }
    if number_of_random_points is not None:
        params["number_of_random_points"] = number_of_random_points
    if standard_deviation_displacement_field is not None:
        params["standard_deviation_displacement_field"] = standard_deviation_displacement_field
    if enforce_stationary_boundary is not None:
        params["enforce_stationary_boundary"] = enforce_stationary_boundary
    if displacement_specific_options is not None:
        params["displacement_specific_options"] = displacement_specific_options
    return params


def simulate_displacement_field_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SimulateDisplacementFieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("displacement_field_type", None) is None:
        raise StyxValidationError("`displacement_field_type` must not be None")
    if not isinstance(params["displacement_field_type"], str):
        raise StyxValidationError(f'`displacement_field_type` has the wrong type: Received `{type(params.get("displacement_field_type", None))}` expected `typing.Literal["BSpline", "Exponential"]`')
    if params["displacement_field_type"] not in ["BSpline", "Exponential"]:
        raise StyxValidationError("Parameter `displacement_field_type` must be one of [\"BSpline\", \"Exponential\"]")
    if params.get("domain_image", None) is None:
        raise StyxValidationError("`domain_image` must not be None")
    if not isinstance(params["domain_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`domain_image` has the wrong type: Received `{type(params.get("domain_image", None))}` expected `InputPathType`')
    if params.get("output_field", None) is None:
        raise StyxValidationError("`output_field` must not be None")
    if not isinstance(params["output_field"], str):
        raise StyxValidationError(f'`output_field` has the wrong type: Received `{type(params.get("output_field", None))}` expected `str`')
    if params.get("number_of_random_points", None) is not None:
        if not isinstance(params["number_of_random_points"], int):
            raise StyxValidationError(f'`number_of_random_points` has the wrong type: Received `{type(params.get("number_of_random_points", None))}` expected `int | None`')
    if params.get("standard_deviation_displacement_field", None) is not None:
        if not isinstance(params["standard_deviation_displacement_field"], (float, int)):
            raise StyxValidationError(f'`standard_deviation_displacement_field` has the wrong type: Received `{type(params.get("standard_deviation_displacement_field", None))}` expected `float | None`')
    if params.get("enforce_stationary_boundary", None) is not None:
        if not isinstance(params["enforce_stationary_boundary"], int):
            raise StyxValidationError(f'`enforce_stationary_boundary` has the wrong type: Received `{type(params.get("enforce_stationary_boundary", None))}` expected `int | None`')
    if params.get("displacement_specific_options", None) is not None:
        if not isinstance(params["displacement_specific_options"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["displacement_specific_options"])}\'')
        if "@type" not in params["displacement_specific_options"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["displacement_specific_options"]["@type"] not in ["bspline_options", "exponential_options"]:
            raise StyxValidationError("Parameter `displacement_specific_options`s `@type` must be one of [\"bspline_options\", \"exponential_options\"]")
        simulate_displacement_field_displacement_specific_options_validate_dyn_fn(params["displacement_specific_options"]["@type"])(params["displacement_specific_options"])


def simulate_displacement_field_cargs(
    params: SimulateDisplacementFieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("SimulateDisplacementField")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(params.get("displacement_field_type", None))
    cargs.append(execution.input_file(params.get("domain_image", None)))
    cargs.append(params.get("output_field", None))
    if params.get("number_of_random_points", None) is not None:
        cargs.append(str(params.get("number_of_random_points", None)))
    if params.get("standard_deviation_displacement_field", None) is not None:
        cargs.append(str(params.get("standard_deviation_displacement_field", None)))
    if params.get("enforce_stationary_boundary", None) is not None:
        cargs.append(str(params.get("enforce_stationary_boundary", None)))
    if params.get("displacement_specific_options", None) is not None:
        cargs.extend(simulate_displacement_field_displacement_specific_options_cargs_dyn_fn(params.get("displacement_specific_options", None)["@type"])(params.get("displacement_specific_options", None), execution))
    return cargs


def simulate_displacement_field_outputs(
    params: SimulateDisplacementFieldParamsDict,
    execution: Execution,
) -> SimulateDisplacementFieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SimulateDisplacementFieldOutputs(
        root=execution.output_file("."),
        output_displacement_field=execution.output_file(params.get("output_field", None)),
    )
    return ret


def simulate_displacement_field_execute(
    params: SimulateDisplacementFieldParamsDict,
    runner: Runner | None = None,
) -> SimulateDisplacementFieldOutputs:
    """
    SimulateDisplacementField
    
    Simulate displacement fields using various methods such as BSpline or
    Exponential.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SimulateDisplacementFieldOutputs`).
    """
    simulate_displacement_field_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SIMULATE_DISPLACEMENT_FIELD_METADATA)
    params = execution.params(params)
    cargs = simulate_displacement_field_cargs(params, execution)
    ret = simulate_displacement_field_outputs(params, execution)
    execution.run(cargs)
    return ret


def simulate_displacement_field(
    image_dimension: int,
    displacement_field_type: typing.Literal["BSpline", "Exponential"],
    domain_image: InputPathType,
    output_field: str,
    number_of_random_points: int | None = None,
    standard_deviation_displacement_field: float | None = None,
    enforce_stationary_boundary: int | None = None,
    displacement_specific_options: typing.Union[SimulateDisplacementFieldBsplineOptionsParamsDictTagged, SimulateDisplacementFieldExponentialOptionsParamsDictTagged] | None = None,
    runner: Runner | None = None,
) -> SimulateDisplacementFieldOutputs:
    """
    SimulateDisplacementField
    
    Simulate displacement fields using various methods such as BSpline or
    Exponential.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimensionality of the image.
        displacement_field_type: Type of displacement field to simulate.
        domain_image: Image defining the domain for the displacement field.
        output_field: Path to save the output displacement field.
        number_of_random_points: Number of random points to use in the\
            simulation.
        standard_deviation_displacement_field: Standard deviation for the\
            displacement field.
        enforce_stationary_boundary: Boolean flag indicating whether to enforce\
            stationary boundary.
        displacement_specific_options: Options specific to the type of\
            displacement field simulation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SimulateDisplacementFieldOutputs`).
    """
    params = simulate_displacement_field_params(
        image_dimension=image_dimension,
        displacement_field_type=displacement_field_type,
        domain_image=domain_image,
        output_field=output_field,
        number_of_random_points=number_of_random_points,
        standard_deviation_displacement_field=standard_deviation_displacement_field,
        enforce_stationary_boundary=enforce_stationary_boundary,
        displacement_specific_options=displacement_specific_options,
    )
    return simulate_displacement_field_execute(params, runner)


__all__ = [
    "SIMULATE_DISPLACEMENT_FIELD_METADATA",
    "SimulateDisplacementFieldBsplineOptionsParamsDict",
    "SimulateDisplacementFieldBsplineOptionsParamsDictTagged",
    "SimulateDisplacementFieldExponentialOptionsParamsDict",
    "SimulateDisplacementFieldExponentialOptionsParamsDictTagged",
    "SimulateDisplacementFieldOutputs",
    "SimulateDisplacementFieldParamsDict",
    "SimulateDisplacementFieldParamsDictTagged",
    "simulate_displacement_field",
    "simulate_displacement_field_bspline_options",
    "simulate_displacement_field_execute",
    "simulate_displacement_field_exponential_options",
    "simulate_displacement_field_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA = Metadata(
    id="4c7c5492fcbe6ca2173159c5be98ecb63525e76c.boutiques",
    name="antsIntermodalityIntrasubject.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsIntermodalityIntrasubjectShParamsDictNoTag = typing.TypedDict('_AntsIntermodalityIntrasubjectShParamsDictNoTag', {
    "dimension": int,
    "anatomical_t1_image": InputPathType,
    "anatomical_reference_image": typing.NotRequired[InputPathType | None],
    "scalar_image_to_match": InputPathType,
    "anatomical_t1brainmask": InputPathType,
    "transform_type": typing.Literal[0, 1, 2, 3],
    "t1_to_template_prefix": str,
    "template_space": typing.NotRequired[str | None],
    "output_prefix": str,
    "labels_in_template_space": typing.NotRequired[InputPathType | None],
    "auxiliary_scalar_images": typing.NotRequired[InputPathType | None],
    "auxiliary_dt_image": typing.NotRequired[InputPathType | None],
})
AntsIntermodalityIntrasubjectShParamsDictTagged = typing.TypedDict('AntsIntermodalityIntrasubjectShParamsDictTagged', {
    "@type": typing.Literal["ants/antsIntermodalityIntrasubject.sh"],
    "dimension": int,
    "anatomical_t1_image": InputPathType,
    "anatomical_reference_image": typing.NotRequired[InputPathType | None],
    "scalar_image_to_match": InputPathType,
    "anatomical_t1brainmask": InputPathType,
    "transform_type": typing.Literal[0, 1, 2, 3],
    "t1_to_template_prefix": str,
    "template_space": typing.NotRequired[str | None],
    "output_prefix": str,
    "labels_in_template_space": typing.NotRequired[InputPathType | None],
    "auxiliary_scalar_images": typing.NotRequired[InputPathType | None],
    "auxiliary_dt_image": typing.NotRequired[InputPathType | None],
})
AntsIntermodalityIntrasubjectShParamsDict = _AntsIntermodalityIntrasubjectShParamsDictNoTag | AntsIntermodalityIntrasubjectShParamsDictTagged


class AntsIntermodalityIntrasubjectShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsIntermodalityIntrasubjectShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transformed_image: OutputPathType
    """Output transformed image after registration."""
    output_transform: OutputPathType
    """Transformation matrix or warp field from the registration."""
    output_warped_image: OutputPathType
    """Output warped image in the template space."""


def ants_intermodality_intrasubject_sh_params(
    dimension: int,
    anatomical_t1_image: InputPathType,
    scalar_image_to_match: InputPathType,
    anatomical_t1brainmask: InputPathType,
    transform_type: typing.Literal[0, 1, 2, 3],
    t1_to_template_prefix: str,
    output_prefix: str,
    anatomical_reference_image: InputPathType | None = None,
    template_space: str | None = None,
    labels_in_template_space: InputPathType | None = None,
    auxiliary_scalar_images: InputPathType | None = None,
    auxiliary_dt_image: InputPathType | None = None,
) -> AntsIntermodalityIntrasubjectShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimension: Dimensionality of the image, typically 3 for 3D images.
        anatomical_t1_image: Anatomical T1 image (brain or whole-head) to align\
            to.
        scalar_image_to_match: Scalar image to be matched, such as average\
            BOLD, average DWI, etc.
        anatomical_t1brainmask: Brain mask for the anatomical T1 image, should\
            mask out regions not appearing in the scalar image.
        transform_type: Type of transform: 0=rigid, 1=affine,\
            2=rigid+small_def, 3=affine+small_def.
        t1_to_template_prefix: Prefix for T1 to template transform files.
        output_prefix: Prefix for output files.
        anatomical_reference_image: Anatomical reference image to warp to,\
            often higher resolution than the anatomical T1 image.
        template_space: Template space.
        labels_in_template_space: Labels in the template space.
        auxiliary_scalar_images: Auxiliary scalar images to warp to the\
            template.
        auxiliary_dt_image: Auxiliary DT image to warp to the template.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsIntermodalityIntrasubject.sh",
        "dimension": dimension,
        "anatomical_t1_image": anatomical_t1_image,
        "scalar_image_to_match": scalar_image_to_match,
        "anatomical_t1brainmask": anatomical_t1brainmask,
        "transform_type": transform_type,
        "t1_to_template_prefix": t1_to_template_prefix,
        "output_prefix": output_prefix,
    }
    if anatomical_reference_image is not None:
        params["anatomical_reference_image"] = anatomical_reference_image
    if template_space is not None:
        params["template_space"] = template_space
    if labels_in_template_space is not None:
        params["labels_in_template_space"] = labels_in_template_space
    if auxiliary_scalar_images is not None:
        params["auxiliary_scalar_images"] = auxiliary_scalar_images
    if auxiliary_dt_image is not None:
        params["auxiliary_dt_image"] = auxiliary_dt_image
    return params


def ants_intermodality_intrasubject_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsIntermodalityIntrasubjectShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimension", None) is None:
        raise StyxValidationError("`dimension` must not be None")
    if not isinstance(params["dimension"], int):
        raise StyxValidationError(f'`dimension` has the wrong type: Received `{type(params.get("dimension", None))}` expected `int`')
    if params.get("anatomical_t1_image", None) is None:
        raise StyxValidationError("`anatomical_t1_image` must not be None")
    if not isinstance(params["anatomical_t1_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical_t1_image` has the wrong type: Received `{type(params.get("anatomical_t1_image", None))}` expected `InputPathType`')
    if params.get("anatomical_reference_image", None) is not None:
        if not isinstance(params["anatomical_reference_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`anatomical_reference_image` has the wrong type: Received `{type(params.get("anatomical_reference_image", None))}` expected `InputPathType | None`')
    if params.get("scalar_image_to_match", None) is None:
        raise StyxValidationError("`scalar_image_to_match` must not be None")
    if not isinstance(params["scalar_image_to_match"], (pathlib.Path, str)):
        raise StyxValidationError(f'`scalar_image_to_match` has the wrong type: Received `{type(params.get("scalar_image_to_match", None))}` expected `InputPathType`')
    if params.get("anatomical_t1brainmask", None) is None:
        raise StyxValidationError("`anatomical_t1brainmask` must not be None")
    if not isinstance(params["anatomical_t1brainmask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical_t1brainmask` has the wrong type: Received `{type(params.get("anatomical_t1brainmask", None))}` expected `InputPathType`')
    if params.get("transform_type", None) is None:
        raise StyxValidationError("`transform_type` must not be None")
    if not isinstance(params["transform_type"], int):
        raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal[0, 1, 2, 3]`')
    if params["transform_type"] not in [0, 1, 2, 3]:
        raise StyxValidationError("Parameter `transform_type` must be one of [0, 1, 2, 3]")
    if params.get("t1_to_template_prefix", None) is None:
        raise StyxValidationError("`t1_to_template_prefix` must not be None")
    if not isinstance(params["t1_to_template_prefix"], str):
        raise StyxValidationError(f'`t1_to_template_prefix` has the wrong type: Received `{type(params.get("t1_to_template_prefix", None))}` expected `str`')
    if params.get("template_space", None) is not None:
        if not isinstance(params["template_space"], str):
            raise StyxValidationError(f'`template_space` has the wrong type: Received `{type(params.get("template_space", None))}` expected `str | None`')
    if params.get("output_prefix", None) is None:
        raise StyxValidationError("`output_prefix` must not be None")
    if not isinstance(params["output_prefix"], str):
        raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str`')
    if params.get("labels_in_template_space", None) is not None:
        if not isinstance(params["labels_in_template_space"], (pathlib.Path, str)):
            raise StyxValidationError(f'`labels_in_template_space` has the wrong type: Received `{type(params.get("labels_in_template_space", None))}` expected `InputPathType | None`')
    if params.get("auxiliary_scalar_images", None) is not None:
        if not isinstance(params["auxiliary_scalar_images"], (pathlib.Path, str)):
            raise StyxValidationError(f'`auxiliary_scalar_images` has the wrong type: Received `{type(params.get("auxiliary_scalar_images", None))}` expected `InputPathType | None`')
    if params.get("auxiliary_dt_image", None) is not None:
        if not isinstance(params["auxiliary_dt_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`auxiliary_dt_image` has the wrong type: Received `{type(params.get("auxiliary_dt_image", None))}` expected `InputPathType | None`')


def ants_intermodality_intrasubject_sh_cargs(
    params: AntsIntermodalityIntrasubjectShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsIntermodalityIntrasubject.sh")
    cargs.extend([
        "-d",
        str(params.get("dimension", None))
    ])
    cargs.extend([
        "-r",
        execution.input_file(params.get("anatomical_t1_image", None))
    ])
    if params.get("anatomical_reference_image", None) is not None:
        cargs.extend([
            "-R",
            execution.input_file(params.get("anatomical_reference_image", None))
        ])
    cargs.extend([
        "-i",
        execution.input_file(params.get("scalar_image_to_match", None))
    ])
    cargs.extend([
        "-x",
        execution.input_file(params.get("anatomical_t1brainmask", None))
    ])
    cargs.extend([
        "-t",
        str(params.get("transform_type", None))
    ])
    cargs.extend([
        "-w",
        params.get("t1_to_template_prefix", None)
    ])
    if params.get("template_space", None) is not None:
        cargs.extend([
            "-T",
            params.get("template_space", None)
        ])
    cargs.extend([
        "-o",
        params.get("output_prefix", None)
    ])
    if params.get("labels_in_template_space", None) is not None:
        cargs.extend([
            "-l",
            execution.input_file(params.get("labels_in_template_space", None))
        ])
    if params.get("auxiliary_scalar_images", None) is not None:
        cargs.extend([
            "-a",
            execution.input_file(params.get("auxiliary_scalar_images", None))
        ])
    if params.get("auxiliary_dt_image", None) is not None:
        cargs.extend([
            "-b",
            execution.input_file(params.get("auxiliary_dt_image", None))
        ])
    return cargs


def ants_intermodality_intrasubject_sh_outputs(
    params: AntsIntermodalityIntrasubjectShParamsDict,
    execution: Execution,
) -> AntsIntermodalityIntrasubjectShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsIntermodalityIntrasubjectShOutputs(
        root=execution.output_file("."),
        output_transformed_image=execution.output_file(params.get("output_prefix", None) + "Transformed.nii.gz"),
        output_transform=execution.output_file(params.get("output_prefix", None) + "Transform.mat"),
        output_warped_image=execution.output_file(params.get("output_prefix", None) + "Warped.nii.gz"),
    )
    return ret


def ants_intermodality_intrasubject_sh_execute(
    params: AntsIntermodalityIntrasubjectShParamsDict,
    runner: Runner | None = None,
) -> AntsIntermodalityIntrasubjectShOutputs:
    """
    antsIntermodalityIntrasubject.sh
    
    Performs registration between a scalar image and a T1 image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsIntermodalityIntrasubjectShOutputs`).
    """
    ants_intermodality_intrasubject_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA)
    params = execution.params(params)
    cargs = ants_intermodality_intrasubject_sh_cargs(params, execution)
    ret = ants_intermodality_intrasubject_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_intermodality_intrasubject_sh(
    dimension: int,
    anatomical_t1_image: InputPathType,
    scalar_image_to_match: InputPathType,
    anatomical_t1brainmask: InputPathType,
    transform_type: typing.Literal[0, 1, 2, 3],
    t1_to_template_prefix: str,
    output_prefix: str,
    anatomical_reference_image: InputPathType | None = None,
    template_space: str | None = None,
    labels_in_template_space: InputPathType | None = None,
    auxiliary_scalar_images: InputPathType | None = None,
    auxiliary_dt_image: InputPathType | None = None,
    runner: Runner | None = None,
) -> AntsIntermodalityIntrasubjectShOutputs:
    """
    antsIntermodalityIntrasubject.sh
    
    Performs registration between a scalar image and a T1 image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: Dimensionality of the image, typically 3 for 3D images.
        anatomical_t1_image: Anatomical T1 image (brain or whole-head) to align\
            to.
        scalar_image_to_match: Scalar image to be matched, such as average\
            BOLD, average DWI, etc.
        anatomical_t1brainmask: Brain mask for the anatomical T1 image, should\
            mask out regions not appearing in the scalar image.
        transform_type: Type of transform: 0=rigid, 1=affine,\
            2=rigid+small_def, 3=affine+small_def.
        t1_to_template_prefix: Prefix for T1 to template transform files.
        output_prefix: Prefix for output files.
        anatomical_reference_image: Anatomical reference image to warp to,\
            often higher resolution than the anatomical T1 image.
        template_space: Template space.
        labels_in_template_space: Labels in the template space.
        auxiliary_scalar_images: Auxiliary scalar images to warp to the\
            template.
        auxiliary_dt_image: Auxiliary DT image to warp to the template.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsIntermodalityIntrasubjectShOutputs`).
    """
    params = ants_intermodality_intrasubject_sh_params(
        dimension=dimension,
        anatomical_t1_image=anatomical_t1_image,
        anatomical_reference_image=anatomical_reference_image,
        scalar_image_to_match=scalar_image_to_match,
        anatomical_t1brainmask=anatomical_t1brainmask,
        transform_type=transform_type,
        t1_to_template_prefix=t1_to_template_prefix,
        template_space=template_space,
        output_prefix=output_prefix,
        labels_in_template_space=labels_in_template_space,
        auxiliary_scalar_images=auxiliary_scalar_images,
        auxiliary_dt_image=auxiliary_dt_image,
    )
    return ants_intermodality_intrasubject_sh_execute(params, runner)


__all__ = [
    "ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA",
    "AntsIntermodalityIntrasubjectShOutputs",
    "AntsIntermodalityIntrasubjectShParamsDict",
    "AntsIntermodalityIntrasubjectShParamsDictTagged",
    "ants_intermodality_intrasubject_sh",
    "ants_intermodality_intrasubject_sh_execute",
    "ants_intermodality_intrasubject_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_TRANSFORM_INFO_METADATA = Metadata(
    id="28abd0b38cba101254396942e6a9c4a39383ae1d.boutiques",
    name="antsTransformInfo",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsTransformInfoParamsDictNoTag = typing.TypedDict('_AntsTransformInfoParamsDictNoTag', {
    "transform_file": InputPathType,
})
AntsTransformInfoParamsDictTagged = typing.TypedDict('AntsTransformInfoParamsDictTagged', {
    "@type": typing.Literal["ants/antsTransformInfo"],
    "transform_file": InputPathType,
})
AntsTransformInfoParamsDict = _AntsTransformInfoParamsDictNoTag | AntsTransformInfoParamsDictTagged


class AntsTransformInfoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsTransformInfoParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_info: OutputPathType
    """Information of the provided transform file."""


def ants_transform_info_params(
    transform_file: InputPathType,
) -> AntsTransformInfoParamsDictTagged:
    """
    Build parameters.
    
    Args:
        transform_file: The transform file to read. Supported formats include\
            HDF5, MINC, Matlab, and Txt.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsTransformInfo",
        "transform_file": transform_file,
    }
    return params


def ants_transform_info_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsTransformInfoParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("transform_file", None) is None:
        raise StyxValidationError("`transform_file` must not be None")
    if not isinstance(params["transform_file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`transform_file` has the wrong type: Received `{type(params.get("transform_file", None))}` expected `InputPathType`')


def ants_transform_info_cargs(
    params: AntsTransformInfoParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsTransformInfo")
    cargs.extend([
        "--file",
        execution.input_file(params.get("transform_file", None))
    ])
    return cargs


def ants_transform_info_outputs(
    params: AntsTransformInfoParamsDict,
    execution: Execution,
) -> AntsTransformInfoOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsTransformInfoOutputs(
        root=execution.output_file("."),
        output_info=execution.output_file("transform_info.txt"),
    )
    return ret


def ants_transform_info_execute(
    params: AntsTransformInfoParamsDict,
    runner: Runner | None = None,
) -> AntsTransformInfoOutputs:
    """
    antsTransformInfo
    
    Provide information about an ITK transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsTransformInfoOutputs`).
    """
    ants_transform_info_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_TRANSFORM_INFO_METADATA)
    params = execution.params(params)
    cargs = ants_transform_info_cargs(params, execution)
    ret = ants_transform_info_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_transform_info(
    transform_file: InputPathType,
    runner: Runner | None = None,
) -> AntsTransformInfoOutputs:
    """
    antsTransformInfo
    
    Provide information about an ITK transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        transform_file: The transform file to read. Supported formats include\
            HDF5, MINC, Matlab, and Txt.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsTransformInfoOutputs`).
    """
    params = ants_transform_info_params(
        transform_file=transform_file,
    )
    return ants_transform_info_execute(params, runner)


__all__ = [
    "ANTS_TRANSFORM_INFO_METADATA",
    "AntsTransformInfoOutputs",
    "AntsTransformInfoParamsDict",
    "AntsTransformInfoParamsDictTagged",
    "ants_transform_info",
    "ants_transform_info_execute",
    "ants_transform_info_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSUSE_DEFORMATION_FIELD_TO_GET_AFFINE_TRANSFORM_METADATA = Metadata(
    id="33730edc7d31f068b95a970d8aa4ca5ece2c7a6f.boutiques",
    name="ANTSUseDeformationFieldToGetAffineTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsuseDeformationFieldToGetAffineTransformParamsDictNoTag = typing.TypedDict('_AntsuseDeformationFieldToGetAffineTransformParamsDictNoTag', {
    "deformation_field": InputPathType,
    "load_ratio": float,
    "transform_type": typing.Literal["rigid", "affine"],
    "output_affine": str,
    "mask": typing.NotRequired[InputPathType | None],
})
AntsuseDeformationFieldToGetAffineTransformParamsDictTagged = typing.TypedDict('AntsuseDeformationFieldToGetAffineTransformParamsDictTagged', {
    "@type": typing.Literal["ants/ANTSUseDeformationFieldToGetAffineTransform"],
    "deformation_field": InputPathType,
    "load_ratio": float,
    "transform_type": typing.Literal["rigid", "affine"],
    "output_affine": str,
    "mask": typing.NotRequired[InputPathType | None],
})
AntsuseDeformationFieldToGetAffineTransformParamsDict = _AntsuseDeformationFieldToGetAffineTransformParamsDictNoTag | AntsuseDeformationFieldToGetAffineTransformParamsDictTagged


class AntsuseDeformationFieldToGetAffineTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsuseDeformationFieldToGetAffineTransformParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_affine_txt: OutputPathType
    """The output is the affine transformation matrix file."""


def antsuse_deformation_field_to_get_affine_transform_params(
    deformation_field: InputPathType,
    load_ratio: float,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
    mask: InputPathType | None = None,
) -> AntsuseDeformationFieldToGetAffineTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        deformation_field: The input deformation field in NIfTI format (e.g.,\
            zzzWarp.nii.gz).
        load_ratio: Ratio of points to be loaded from deformation field to save\
            memory (ex: 0.01).
        transform_type: Type of transform to be extracted. Can be 'rigid' or\
            'affine'.
        output_affine: The output file where the affine transform will be saved\
            (e.g., OutAffine.txt).
        mask: Optional mask file defining the region from which points will be\
            selected.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ANTSUseDeformationFieldToGetAffineTransform",
        "deformation_field": deformation_field,
        "load_ratio": load_ratio,
        "transform_type": transform_type,
        "output_affine": output_affine,
    }
    if mask is not None:
        params["mask"] = mask
    return params


def antsuse_deformation_field_to_get_affine_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsuseDeformationFieldToGetAffineTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("deformation_field", None) is None:
        raise StyxValidationError("`deformation_field` must not be None")
    if not isinstance(params["deformation_field"], (pathlib.Path, str)):
        raise StyxValidationError(f'`deformation_field` has the wrong type: Received `{type(params.get("deformation_field", None))}` expected `InputPathType`')
    if params.get("load_ratio", None) is None:
        raise StyxValidationError("`load_ratio` must not be None")
    if not isinstance(params["load_ratio"], (float, int)):
        raise StyxValidationError(f'`load_ratio` has the wrong type: Received `{type(params.get("load_ratio", None))}` expected `float`')
    if params.get("transform_type", None) is None:
        raise StyxValidationError("`transform_type` must not be None")
    if not isinstance(params["transform_type"], str):
        raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal["rigid", "affine"]`')
    if params["transform_type"] not in ["rigid", "affine"]:
        raise StyxValidationError("Parameter `transform_type` must be one of [\"rigid\", \"affine\"]")
    if params.get("output_affine", None) is None:
        raise StyxValidationError("`output_affine` must not be None")
    if not isinstance(params["output_affine"], str):
        raise StyxValidationError(f'`output_affine` has the wrong type: Received `{type(params.get("output_affine", None))}` expected `str`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')


def antsuse_deformation_field_to_get_affine_transform_cargs(
    params: AntsuseDeformationFieldToGetAffineTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSUseDeformationFieldToGetAffineTransform")
    cargs.append(execution.input_file(params.get("deformation_field", None)))
    cargs.append(str(params.get("load_ratio", None)))
    cargs.append(params.get("transform_type", None))
    cargs.append(params.get("output_affine", None))
    if params.get("mask", None) is not None:
        cargs.append(execution.input_file(params.get("mask", None)))
    return cargs


def antsuse_deformation_field_to_get_affine_transform_outputs(
    params: AntsuseDeformationFieldToGetAffineTransformParamsDict,
    execution: Execution,
) -> AntsuseDeformationFieldToGetAffineTransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsuseDeformationFieldToGetAffineTransformOutputs(
        root=execution.output_file("."),
        out_affine_txt=execution.output_file(params.get("output_affine", None)),
    )
    return ret


def antsuse_deformation_field_to_get_affine_transform_execute(
    params: AntsuseDeformationFieldToGetAffineTransformParamsDict,
    runner: Runner | None = None,
) -> AntsuseDeformationFieldToGetAffineTransformOutputs:
    """
    ANTSUseDeformationFieldToGetAffineTransform
    
    Extracts an affine transform from a deformation field. The input deformation
    field is expected to be in the same physical space as the images you want to
    transform.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseDeformationFieldToGetAffineTransformOutputs`).
    """
    antsuse_deformation_field_to_get_affine_transform_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSUSE_DEFORMATION_FIELD_TO_GET_AFFINE_TRANSFORM_METADATA)
    params = execution.params(params)
    cargs = antsuse_deformation_field_to_get_affine_transform_cargs(params, execution)
    ret = antsuse_deformation_field_to_get_affine_transform_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsuse_deformation_field_to_get_affine_transform(
    deformation_field: InputPathType,
    load_ratio: float,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
    mask: InputPathType | None = None,
    runner: Runner | None = None,
) -> AntsuseDeformationFieldToGetAffineTransformOutputs:
    """
    ANTSUseDeformationFieldToGetAffineTransform
    
    Extracts an affine transform from a deformation field. The input deformation
    field is expected to be in the same physical space as the images you want to
    transform.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        deformation_field: The input deformation field in NIfTI format (e.g.,\
            zzzWarp.nii.gz).
        load_ratio: Ratio of points to be loaded from deformation field to save\
            memory (ex: 0.01).
        transform_type: Type of transform to be extracted. Can be 'rigid' or\
            'affine'.
        output_affine: The output file where the affine transform will be saved\
            (e.g., OutAffine.txt).
        mask: Optional mask file defining the region from which points will be\
            selected.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseDeformationFieldToGetAffineTransformOutputs`).
    """
    params = antsuse_deformation_field_to_get_affine_transform_params(
        deformation_field=deformation_field,
        load_ratio=load_ratio,
        transform_type=transform_type,
        output_affine=output_affine,
        mask=mask,
    )
    return antsuse_deformation_field_to_get_affine_transform_execute(params, runner)


__all__ = [
    "ANTSUSE_DEFORMATION_FIELD_TO_GET_AFFINE_TRANSFORM_METADATA",
    "AntsuseDeformationFieldToGetAffineTransformOutputs",
    "AntsuseDeformationFieldToGetAffineTransformParamsDict",
    "AntsuseDeformationFieldToGetAffineTransformParamsDictTagged",
    "antsuse_deformation_field_to_get_affine_transform",
    "antsuse_deformation_field_to_get_affine_transform_execute",
    "antsuse_deformation_field_to_get_affine_transform_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CREATE_TILED_MOSAIC_METADATA = Metadata(
    id="ce3fc57fea426a71907429b635a0ccf43f34f6db.boutiques",
    name="CreateTiledMosaic",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_CreateTiledMosaicParamsDictNoTag = typing.TypedDict('_CreateTiledMosaicParamsDictNoTag', {
    "input_image": InputPathType,
    "rgb_image": typing.NotRequired[InputPathType | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "alpha": typing.NotRequired[float | None],
    "functional_overlay": typing.NotRequired[str | None],
    "output": str,
    "tile_geometry": typing.NotRequired[str | None],
    "direction": typing.NotRequired[typing.Literal["0", "1", "2", "x", "y", "z"] | None],
    "pad_or_crop": typing.NotRequired[str | None],
    "slices": typing.NotRequired[str | None],
    "flip_slice": typing.NotRequired[str | None],
    "permute_axes": typing.NotRequired[bool | None],
})
CreateTiledMosaicParamsDictTagged = typing.TypedDict('CreateTiledMosaicParamsDictTagged', {
    "@type": typing.Literal["ants/CreateTiledMosaic"],
    "input_image": InputPathType,
    "rgb_image": typing.NotRequired[InputPathType | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "alpha": typing.NotRequired[float | None],
    "functional_overlay": typing.NotRequired[str | None],
    "output": str,
    "tile_geometry": typing.NotRequired[str | None],
    "direction": typing.NotRequired[typing.Literal["0", "1", "2", "x", "y", "z"] | None],
    "pad_or_crop": typing.NotRequired[str | None],
    "slices": typing.NotRequired[str | None],
    "flip_slice": typing.NotRequired[str | None],
    "permute_axes": typing.NotRequired[bool | None],
})
CreateTiledMosaicParamsDict = _CreateTiledMosaicParamsDictNoTag | CreateTiledMosaicParamsDictTagged


class CreateTiledMosaicOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateTiledMosaicParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    tiled_mosaic_image: OutputPathType
    """The output is the tiled mosaic image."""


def create_tiled_mosaic_params(
    input_image: InputPathType,
    output: str,
    rgb_image: InputPathType | None = None,
    mask_image: InputPathType | None = None,
    alpha: float | None = None,
    functional_overlay: str | None = None,
    tile_geometry: str | None = None,
    direction: typing.Literal["0", "1", "2", "x", "y", "z"] | None = None,
    pad_or_crop: str | None = None,
    slices: str | None = None,
    flip_slice: str | None = None,
    permute_axes: bool | None = None,
) -> CreateTiledMosaicParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_image: Main input is a 3-D grayscale image.
        output: The output is the tiled mosaic image. The format must support\
            the specific data type: floating point images without RGB overlays, Rgb\
            images with intensities scaled to [0,255] if overlays are present.
        rgb_image: An optional Rgb image can be added as an overlay. It must\
            have the same image geometry as the input grayscale image.
        mask_image: Specifies the ROI of the RGB voxels used.
        alpha: If an Rgb image is provided, render the overlay using the\
            specified alpha parameter.
        functional_overlay: A functional overlay can be specified using both\
            and rgb image and a mask specifying where that rgb image should be\
            applied. Both images must have the same image geometry as the input\
            image. Optionally, an alpha parameter can be specified.
        tile_geometry: The tile geometry specifies the number of rows and\
            columns in the output image. For example, specifying '5x10' renders 5\
            rows by 10 columns of slices.
        direction: Specifies the direction of the slices. Can be based on image\
            storage in memory or aligned physical space. Defaults to z-direction if\
            unspecified.
        pad_or_crop: Specify padding or cropping with a voxel-width boundary\
            for each slice. Padding uses a specified constant value. Cropping pads\
            with negative voxel-widths.
        slices: Control over which slices to render. Specify slices directly or\
            incrementally with optional start and end slices.
        flip_slice: Flip individual slice images horizontally and/or\
            vertically.
        permute_axes: Permute (or swap) the axes of the individual slice\
            images.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/CreateTiledMosaic",
        "input_image": input_image,
        "output": output,
    }
    if rgb_image is not None:
        params["rgb_image"] = rgb_image
    if mask_image is not None:
        params["mask_image"] = mask_image
    if alpha is not None:
        params["alpha"] = alpha
    if functional_overlay is not None:
        params["functional_overlay"] = functional_overlay
    if tile_geometry is not None:
        params["tile_geometry"] = tile_geometry
    if direction is not None:
        params["direction"] = direction
    if pad_or_crop is not None:
        params["pad_or_crop"] = pad_or_crop
    if slices is not None:
        params["slices"] = slices
    if flip_slice is not None:
        params["flip_slice"] = flip_slice
    if permute_axes is not None:
        params["permute_axes"] = permute_axes
    return params


def create_tiled_mosaic_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CreateTiledMosaicParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("rgb_image", None) is not None:
        if not isinstance(params["rgb_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`rgb_image` has the wrong type: Received `{type(params.get("rgb_image", None))}` expected `InputPathType | None`')
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("alpha", None) is not None:
        if not isinstance(params["alpha"], (float, int)):
            raise StyxValidationError(f'`alpha` has the wrong type: Received `{type(params.get("alpha", None))}` expected `float | None`')
    if params.get("functional_overlay", None) is not None:
        if not isinstance(params["functional_overlay"], str):
            raise StyxValidationError(f'`functional_overlay` has the wrong type: Received `{type(params.get("functional_overlay", None))}` expected `str | None`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("tile_geometry", None) is not None:
        if not isinstance(params["tile_geometry"], str):
            raise StyxValidationError(f'`tile_geometry` has the wrong type: Received `{type(params.get("tile_geometry", None))}` expected `str | None`')
    if params.get("direction", None) is not None:
        if not isinstance(params["direction"], str):
            raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `typing.Literal["0", "1", "2", "x", "y", "z"] | None`')
        if params["direction"] not in ["0", "1", "2", "x", "y", "z"]:
            raise StyxValidationError("Parameter `direction` must be one of [\"0\", \"1\", \"2\", \"x\", \"y\", \"z\"]")
    if params.get("pad_or_crop", None) is not None:
        if not isinstance(params["pad_or_crop"], str):
            raise StyxValidationError(f'`pad_or_crop` has the wrong type: Received `{type(params.get("pad_or_crop", None))}` expected `str | None`')
    if params.get("slices", None) is not None:
        if not isinstance(params["slices"], str):
            raise StyxValidationError(f'`slices` has the wrong type: Received `{type(params.get("slices", None))}` expected `str | None`')
    if params.get("flip_slice", None) is not None:
        if not isinstance(params["flip_slice"], str):
            raise StyxValidationError(f'`flip_slice` has the wrong type: Received `{type(params.get("flip_slice", None))}` expected `str | None`')
    if params.get("permute_axes", None) is not None:
        if not isinstance(params["permute_axes"], bool):
            raise StyxValidationError(f'`permute_axes` has the wrong type: Received `{type(params.get("permute_axes", None))}` expected `bool | None`')


def create_tiled_mosaic_cargs(
    params: CreateTiledMosaicParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("CreateTiledMosaic")
    cargs.extend([
        "-i",
        execution.input_file(params.get("input_image", None))
    ])
    if params.get("rgb_image", None) is not None:
        cargs.extend([
            "-r",
            execution.input_file(params.get("rgb_image", None))
        ])
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "-x",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("alpha", None) is not None:
        cargs.extend([
            "-a",
            str(params.get("alpha", None))
        ])
    if params.get("functional_overlay", None) is not None:
        cargs.extend([
            "-e",
            params.get("functional_overlay", None)
        ])
    cargs.extend([
        "-o",
        params.get("output", None)
    ])
    if params.get("tile_geometry", None) is not None:
        cargs.extend([
            "-t",
            params.get("tile_geometry", None)
        ])
    if params.get("direction", None) is not None:
        cargs.extend([
            "-d",
            params.get("direction", None)
        ])
    if params.get("pad_or_crop", None) is not None:
        cargs.extend([
            "-p",
            params.get("pad_or_crop", None)
        ])
    if params.get("slices", None) is not None:
        cargs.extend([
            "-s",
            params.get("slices", None)
        ])
    if params.get("flip_slice", None) is not None:
        cargs.extend([
            "-f",
            params.get("flip_slice", None)
        ])
    if params.get("permute_axes", None) is not None:
        cargs.extend([
            "-g",
            ("1" if params.get("permute_axes", None) else "0")
        ])
    return cargs


def create_tiled_mosaic_outputs(
    params: CreateTiledMosaicParamsDict,
    execution: Execution,
) -> CreateTiledMosaicOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateTiledMosaicOutputs(
        root=execution.output_file("."),
        tiled_mosaic_image=execution.output_file(params.get("output", None)),
    )
    return ret


def create_tiled_mosaic_execute(
    params: CreateTiledMosaicParamsDict,
    runner: Runner | None = None,
) -> CreateTiledMosaicOutputs:
    """
    CreateTiledMosaic
    
    Render a 3-D image volume with optional Rgb overlay.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateTiledMosaicOutputs`).
    """
    create_tiled_mosaic_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_TILED_MOSAIC_METADATA)
    params = execution.params(params)
    cargs = create_tiled_mosaic_cargs(params, execution)
    ret = create_tiled_mosaic_outputs(params, execution)
    execution.run(cargs)
    return ret


def create_tiled_mosaic(
    input_image: InputPathType,
    output: str,
    rgb_image: InputPathType | None = None,
    mask_image: InputPathType | None = None,
    alpha: float | None = None,
    functional_overlay: str | None = None,
    tile_geometry: str | None = None,
    direction: typing.Literal["0", "1", "2", "x", "y", "z"] | None = None,
    pad_or_crop: str | None = None,
    slices: str | None = None,
    flip_slice: str | None = None,
    permute_axes: bool | None = None,
    runner: Runner | None = None,
) -> CreateTiledMosaicOutputs:
    """
    CreateTiledMosaic
    
    Render a 3-D image volume with optional Rgb overlay.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: Main input is a 3-D grayscale image.
        output: The output is the tiled mosaic image. The format must support\
            the specific data type: floating point images without RGB overlays, Rgb\
            images with intensities scaled to [0,255] if overlays are present.
        rgb_image: An optional Rgb image can be added as an overlay. It must\
            have the same image geometry as the input grayscale image.
        mask_image: Specifies the ROI of the RGB voxels used.
        alpha: If an Rgb image is provided, render the overlay using the\
            specified alpha parameter.
        functional_overlay: A functional overlay can be specified using both\
            and rgb image and a mask specifying where that rgb image should be\
            applied. Both images must have the same image geometry as the input\
            image. Optionally, an alpha parameter can be specified.
        tile_geometry: The tile geometry specifies the number of rows and\
            columns in the output image. For example, specifying '5x10' renders 5\
            rows by 10 columns of slices.
        direction: Specifies the direction of the slices. Can be based on image\
            storage in memory or aligned physical space. Defaults to z-direction if\
            unspecified.
        pad_or_crop: Specify padding or cropping with a voxel-width boundary\
            for each slice. Padding uses a specified constant value. Cropping pads\
            with negative voxel-widths.
        slices: Control over which slices to render. Specify slices directly or\
            incrementally with optional start and end slices.
        flip_slice: Flip individual slice images horizontally and/or\
            vertically.
        permute_axes: Permute (or swap) the axes of the individual slice\
            images.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateTiledMosaicOutputs`).
    """
    params = create_tiled_mosaic_params(
        input_image=input_image,
        rgb_image=rgb_image,
        mask_image=mask_image,
        alpha=alpha,
        functional_overlay=functional_overlay,
        output=output,
        tile_geometry=tile_geometry,
        direction=direction,
        pad_or_crop=pad_or_crop,
        slices=slices,
        flip_slice=flip_slice,
        permute_axes=permute_axes,
    )
    return create_tiled_mosaic_execute(params, runner)


__all__ = [
    "CREATE_TILED_MOSAIC_METADATA",
    "CreateTiledMosaicOutputs",
    "CreateTiledMosaicParamsDict",
    "CreateTiledMosaicParamsDictTagged",
    "create_tiled_mosaic",
    "create_tiled_mosaic_execute",
    "create_tiled_mosaic_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CREATE_WARPED_GRID_IMAGE_METADATA = Metadata(
    id="e77b860defec17e64141d0e139550e9625747193.boutiques",
    name="CreateWarpedGridImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_CreateWarpedGridImageParamsDictNoTag = typing.TypedDict('_CreateWarpedGridImageParamsDictNoTag', {
    "image_dimension": int,
    "deformation_field": InputPathType,
    "output_image": str,
    "directions": typing.NotRequired[str | None],
    "grid_spacing": typing.NotRequired[str | None],
    "grid_sigma": typing.NotRequired[str | None],
})
CreateWarpedGridImageParamsDictTagged = typing.TypedDict('CreateWarpedGridImageParamsDictTagged', {
    "@type": typing.Literal["ants/CreateWarpedGridImage"],
    "image_dimension": int,
    "deformation_field": InputPathType,
    "output_image": str,
    "directions": typing.NotRequired[str | None],
    "grid_spacing": typing.NotRequired[str | None],
    "grid_sigma": typing.NotRequired[str | None],
})
CreateWarpedGridImageParamsDict = _CreateWarpedGridImageParamsDictNoTag | CreateWarpedGridImageParamsDictTagged


class CreateWarpedGridImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateWarpedGridImageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    warped_grid_image: OutputPathType
    """The resultant warped grid image."""


def create_warped_grid_image_params(
    image_dimension: int,
    deformation_field: InputPathType,
    output_image: str,
    directions: str | None = None,
    grid_spacing: str | None = None,
    grid_sigma: str | None = None,
) -> CreateWarpedGridImageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the input image.
        deformation_field: File containing the deformation field to be applied.
        output_image: The filename of the output warped grid image.
        directions: Directions for the grid warping, e.g., '1x0x0'.
        grid_spacing: Spacing of the grid, e.g., '10x10x10'.
        grid_sigma: Sigma value for the grid smoothing, e.g., '1x1x1'.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/CreateWarpedGridImage",
        "image_dimension": image_dimension,
        "deformation_field": deformation_field,
        "output_image": output_image,
    }
    if directions is not None:
        params["directions"] = directions
    if grid_spacing is not None:
        params["grid_spacing"] = grid_spacing
    if grid_sigma is not None:
        params["grid_sigma"] = grid_sigma
    return params


def create_warped_grid_image_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CreateWarpedGridImageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("deformation_field", None) is None:
        raise StyxValidationError("`deformation_field` must not be None")
    if not isinstance(params["deformation_field"], (pathlib.Path, str)):
        raise StyxValidationError(f'`deformation_field` has the wrong type: Received `{type(params.get("deformation_field", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("directions", None) is not None:
        if not isinstance(params["directions"], str):
            raise StyxValidationError(f'`directions` has the wrong type: Received `{type(params.get("directions", None))}` expected `str | None`')
    if params.get("grid_spacing", None) is not None:
        if not isinstance(params["grid_spacing"], str):
            raise StyxValidationError(f'`grid_spacing` has the wrong type: Received `{type(params.get("grid_spacing", None))}` expected `str | None`')
    if params.get("grid_sigma", None) is not None:
        if not isinstance(params["grid_sigma"], str):
            raise StyxValidationError(f'`grid_sigma` has the wrong type: Received `{type(params.get("grid_sigma", None))}` expected `str | None`')


def create_warped_grid_image_cargs(
    params: CreateWarpedGridImageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("CreateWarpedGridImage")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("deformation_field", None)))
    cargs.append(params.get("output_image", None))
    if params.get("directions", None) is not None:
        cargs.append(params.get("directions", None))
    if params.get("grid_spacing", None) is not None:
        cargs.append(params.get("grid_spacing", None))
    if params.get("grid_sigma", None) is not None:
        cargs.append(params.get("grid_sigma", None))
    return cargs


def create_warped_grid_image_outputs(
    params: CreateWarpedGridImageParamsDict,
    execution: Execution,
) -> CreateWarpedGridImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateWarpedGridImageOutputs(
        root=execution.output_file("."),
        warped_grid_image=execution.output_file(params.get("output_image", None)),
    )
    return ret


def create_warped_grid_image_execute(
    params: CreateWarpedGridImageParamsDict,
    runner: Runner | None = None,
) -> CreateWarpedGridImageOutputs:
    """
    CreateWarpedGridImage
    
    Create a warped grid image based on the specified deformation field.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateWarpedGridImageOutputs`).
    """
    create_warped_grid_image_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_WARPED_GRID_IMAGE_METADATA)
    params = execution.params(params)
    cargs = create_warped_grid_image_cargs(params, execution)
    ret = create_warped_grid_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def create_warped_grid_image(
    image_dimension: int,
    deformation_field: InputPathType,
    output_image: str,
    directions: str | None = None,
    grid_spacing: str | None = None,
    grid_sigma: str | None = None,
    runner: Runner | None = None,
) -> CreateWarpedGridImageOutputs:
    """
    CreateWarpedGridImage
    
    Create a warped grid image based on the specified deformation field.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the input image.
        deformation_field: File containing the deformation field to be applied.
        output_image: The filename of the output warped grid image.
        directions: Directions for the grid warping, e.g., '1x0x0'.
        grid_spacing: Spacing of the grid, e.g., '10x10x10'.
        grid_sigma: Sigma value for the grid smoothing, e.g., '1x1x1'.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateWarpedGridImageOutputs`).
    """
    params = create_warped_grid_image_params(
        image_dimension=image_dimension,
        deformation_field=deformation_field,
        output_image=output_image,
        directions=directions,
        grid_spacing=grid_spacing,
        grid_sigma=grid_sigma,
    )
    return create_warped_grid_image_execute(params, runner)


__all__ = [
    "CREATE_WARPED_GRID_IMAGE_METADATA",
    "CreateWarpedGridImageOutputs",
    "CreateWarpedGridImageParamsDict",
    "CreateWarpedGridImageParamsDictTagged",
    "create_warped_grid_image",
    "create_warped_grid_image_execute",
    "create_warped_grid_image_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ATROPOS_METADATA = Metadata(
    id="6436edca90da7aa4163ae647bc1238ae647dd4fd.boutiques",
    name="Atropos",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AtroposParamsDictNoTag = typing.TypedDict('_AtroposParamsDictNoTag', {
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "intensity_image": str,
    "bspline": typing.NotRequired[str | None],
    "initialization": str,
    "partial_volume_label_set": typing.NotRequired[str | None],
    "use_partial_volume_likelihoods": typing.NotRequired[bool | None],
    "posterior_formulation": typing.NotRequired[str | None],
    "mask_image": InputPathType,
    "convergence": str,
    "likelihood_model": str,
    "mrf": typing.NotRequired[str | None],
    "icm": typing.NotRequired[str | None],
    "use_random_seed": typing.NotRequired[bool | None],
    "output": str,
    "minimize_memory_usage": typing.NotRequired[bool | None],
    "winsorize_outliers": typing.NotRequired[str | None],
    "use_euclidean_distance": typing.NotRequired[bool | None],
    "label_propagation": typing.NotRequired[str | None],
    "verbose": typing.NotRequired[bool | None],
})
AtroposParamsDictTagged = typing.TypedDict('AtroposParamsDictTagged', {
    "@type": typing.Literal["ants/Atropos"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "intensity_image": str,
    "bspline": typing.NotRequired[str | None],
    "initialization": str,
    "partial_volume_label_set": typing.NotRequired[str | None],
    "use_partial_volume_likelihoods": typing.NotRequired[bool | None],
    "posterior_formulation": typing.NotRequired[str | None],
    "mask_image": InputPathType,
    "convergence": str,
    "likelihood_model": str,
    "mrf": typing.NotRequired[str | None],
    "icm": typing.NotRequired[str | None],
    "use_random_seed": typing.NotRequired[bool | None],
    "output": str,
    "minimize_memory_usage": typing.NotRequired[bool | None],
    "winsorize_outliers": typing.NotRequired[str | None],
    "use_euclidean_distance": typing.NotRequired[bool | None],
    "label_propagation": typing.NotRequired[str | None],
    "verbose": typing.NotRequired[bool | None],
})
AtroposParamsDict = _AtroposParamsDictNoTag | AtroposParamsDictTagged


class AtroposOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AtroposParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    classified_image: OutputPathType
    """The output labeled image with assigned labels for each voxel in the
    masked region."""
    posterior_probability_images: OutputPathType
    """Output posterior probability images in specified format."""


def atropos_params(
    intensity_image: str,
    initialization: str,
    mask_image: InputPathType,
    convergence: str,
    likelihood_model: str,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    bspline: str | None = None,
    partial_volume_label_set: str | None = None,
    use_partial_volume_likelihoods: bool | None = None,
    posterior_formulation: str | None = None,
    mrf: str | None = None,
    icm: str | None = None,
    use_random_seed: bool | None = None,
    minimize_memory_usage: bool | None = None,
    winsorize_outliers: str | None = None,
    use_euclidean_distance: bool | None = None,
    label_propagation: str | None = None,
    verbose: bool | None = None,
) -> AtroposParamsDictTagged:
    """
    Build parameters.
    
    Args:
        intensity_image: One or more scalar images is specified for\
            segmentation. For scenarios with no prior information, the first scalar\
            image is used to order labelings by intensity. The optional adaptive\
            smoothing weight is applicable with prior images, specified between\
            [0,1].
        initialization: Initialize the FMM parameters. options include Random,\
            Otsu, KMeans, PriorProbabilityImages, and PriorLabelImage.
        mask_image: The required image mask defines the region to be labeled by\
            Atropos.
        convergence: Determine convergence based on mean maximum posterior\
            probability over region of interest.
        likelihood_model: Specify parametric or non-parametric likelihood\
            model. Options include Gaussian, HistogramParzenWindows,\
            ManifoldParzenWindows, among others.
        output: Output labeled image and optionally posterior probability\
            images.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, Atropos tries to infer\
            the dimensionality from the first input image.
        bspline: Parameters for B-Spline. Adaptive smoothing is applied to\
            intensity images if smoothing weights > 0.
        partial_volume_label_set: Model mixtures of classes within single\
            voxels. Specify labels for each partial volume class.
        use_partial_volume_likelihoods: Whether to use partial volume\
            likelihoods. A value of 1 considers the partial volume class separate\
            from tissue classes.
        posterior_formulation: Specify posterior probability formulation.\
            Options are Socrates, Plato, Aristotle, or Sigmoid.
        mrf: Markov Random Field parameters to enforce spatial constraints on\
            segmentation.
        icm: ICM (Iterated Conditional Modes) parameters for asynchronous\
            updating.
        use_random_seed: Initialize with a random seed or a constant seed\
            number.
        minimize_memory_usage: Minimize memory usage by calculating images on\
            the fly and storing only non-negligible pixel values.
        winsorize_outliers: Options to remove effects of outliers in\
            calculations using methods like BoxPlot or GrubbsRosner.
        use_euclidean_distance: Propagate labels throughout the mask using a\
            distance transform.
        label_propagation: Control propagation of each prior label by specified\
            lambda and boundary probability.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/Atropos",
        "intensity_image": intensity_image,
        "initialization": initialization,
        "mask_image": mask_image,
        "convergence": convergence,
        "likelihood_model": likelihood_model,
        "output": output,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if bspline is not None:
        params["bspline"] = bspline
    if partial_volume_label_set is not None:
        params["partial_volume_label_set"] = partial_volume_label_set
    if use_partial_volume_likelihoods is not None:
        params["use_partial_volume_likelihoods"] = use_partial_volume_likelihoods
    if posterior_formulation is not None:
        params["posterior_formulation"] = posterior_formulation
    if mrf is not None:
        params["mrf"] = mrf
    if icm is not None:
        params["icm"] = icm
    if use_random_seed is not None:
        params["use_random_seed"] = use_random_seed
    if minimize_memory_usage is not None:
        params["minimize_memory_usage"] = minimize_memory_usage
    if winsorize_outliers is not None:
        params["winsorize_outliers"] = winsorize_outliers
    if use_euclidean_distance is not None:
        params["use_euclidean_distance"] = use_euclidean_distance
    if label_propagation is not None:
        params["label_propagation"] = label_propagation
    if verbose is not None:
        params["verbose"] = verbose
    return params


def atropos_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AtroposParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimensionality", None) is not None:
        if not isinstance(params["image_dimensionality"], int):
            raise StyxValidationError(f'`image_dimensionality` has the wrong type: Received `{type(params.get("image_dimensionality", None))}` expected `typing.Literal[2, 3, 4] | None`')
        if params["image_dimensionality"] not in [2, 3, 4]:
            raise StyxValidationError("Parameter `image_dimensionality` must be one of [2, 3, 4]")
    if params.get("intensity_image", None) is None:
        raise StyxValidationError("`intensity_image` must not be None")
    if not isinstance(params["intensity_image"], str):
        raise StyxValidationError(f'`intensity_image` has the wrong type: Received `{type(params.get("intensity_image", None))}` expected `str`')
    if params.get("bspline", None) is not None:
        if not isinstance(params["bspline"], str):
            raise StyxValidationError(f'`bspline` has the wrong type: Received `{type(params.get("bspline", None))}` expected `str | None`')
    if params.get("initialization", None) is None:
        raise StyxValidationError("`initialization` must not be None")
    if not isinstance(params["initialization"], str):
        raise StyxValidationError(f'`initialization` has the wrong type: Received `{type(params.get("initialization", None))}` expected `str`')
    if params.get("partial_volume_label_set", None) is not None:
        if not isinstance(params["partial_volume_label_set"], str):
            raise StyxValidationError(f'`partial_volume_label_set` has the wrong type: Received `{type(params.get("partial_volume_label_set", None))}` expected `str | None`')
    if params.get("use_partial_volume_likelihoods", None) is not None:
        if not isinstance(params["use_partial_volume_likelihoods"], bool):
            raise StyxValidationError(f'`use_partial_volume_likelihoods` has the wrong type: Received `{type(params.get("use_partial_volume_likelihoods", None))}` expected `bool | None`')
    if params.get("posterior_formulation", None) is not None:
        if not isinstance(params["posterior_formulation"], str):
            raise StyxValidationError(f'`posterior_formulation` has the wrong type: Received `{type(params.get("posterior_formulation", None))}` expected `str | None`')
    if params.get("mask_image", None) is None:
        raise StyxValidationError("`mask_image` must not be None")
    if not isinstance(params["mask_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType`')
    if params.get("convergence", None) is None:
        raise StyxValidationError("`convergence` must not be None")
    if not isinstance(params["convergence"], str):
        raise StyxValidationError(f'`convergence` has the wrong type: Received `{type(params.get("convergence", None))}` expected `str`')
    if params.get("likelihood_model", None) is None:
        raise StyxValidationError("`likelihood_model` must not be None")
    if not isinstance(params["likelihood_model"], str):
        raise StyxValidationError(f'`likelihood_model` has the wrong type: Received `{type(params.get("likelihood_model", None))}` expected `str`')
    if params.get("mrf", None) is not None:
        if not isinstance(params["mrf"], str):
            raise StyxValidationError(f'`mrf` has the wrong type: Received `{type(params.get("mrf", None))}` expected `str | None`')
    if params.get("icm", None) is not None:
        if not isinstance(params["icm"], str):
            raise StyxValidationError(f'`icm` has the wrong type: Received `{type(params.get("icm", None))}` expected `str | None`')
    if params.get("use_random_seed", None) is not None:
        if not isinstance(params["use_random_seed"], bool):
            raise StyxValidationError(f'`use_random_seed` has the wrong type: Received `{type(params.get("use_random_seed", None))}` expected `bool | None`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("minimize_memory_usage", None) is not None:
        if not isinstance(params["minimize_memory_usage"], bool):
            raise StyxValidationError(f'`minimize_memory_usage` has the wrong type: Received `{type(params.get("minimize_memory_usage", None))}` expected `bool | None`')
    if params.get("winsorize_outliers", None) is not None:
        if not isinstance(params["winsorize_outliers"], str):
            raise StyxValidationError(f'`winsorize_outliers` has the wrong type: Received `{type(params.get("winsorize_outliers", None))}` expected `str | None`')
    if params.get("use_euclidean_distance", None) is not None:
        if not isinstance(params["use_euclidean_distance"], bool):
            raise StyxValidationError(f'`use_euclidean_distance` has the wrong type: Received `{type(params.get("use_euclidean_distance", None))}` expected `bool | None`')
    if params.get("label_propagation", None) is not None:
        if not isinstance(params["label_propagation"], str):
            raise StyxValidationError(f'`label_propagation` has the wrong type: Received `{type(params.get("label_propagation", None))}` expected `str | None`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')


def atropos_cargs(
    params: AtroposParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Atropos")
    if params.get("image_dimensionality", None) is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality", None))
        ])
    cargs.extend([
        "-a",
        params.get("intensity_image", None)
    ])
    if params.get("bspline", None) is not None:
        cargs.extend([
            "-b",
            params.get("bspline", None)
        ])
    cargs.extend([
        "-i",
        params.get("initialization", None)
    ])
    if params.get("partial_volume_label_set", None) is not None:
        cargs.extend([
            "-s",
            params.get("partial_volume_label_set", None)
        ])
    if params.get("use_partial_volume_likelihoods", None) is not None:
        cargs.extend([
            "--use-partial-volume-likelihoods",
            ("1" if params.get("use_partial_volume_likelihoods", None) else "0")
        ])
    if params.get("posterior_formulation", None) is not None:
        cargs.extend([
            "-p",
            params.get("posterior_formulation", None)
        ])
    cargs.extend([
        "-x",
        execution.input_file(params.get("mask_image", None))
    ])
    cargs.extend([
        "-c",
        params.get("convergence", None)
    ])
    cargs.extend([
        "-k",
        params.get("likelihood_model", None)
    ])
    if params.get("mrf", None) is not None:
        cargs.extend([
            "-m",
            params.get("mrf", None)
        ])
    if params.get("icm", None) is not None:
        cargs.extend([
            "-g",
            params.get("icm", None)
        ])
    if params.get("use_random_seed", None) is not None:
        cargs.extend([
            "-r",
            ("1" if params.get("use_random_seed", None) else "0")
        ])
    cargs.extend([
        "-o",
        params.get("output", None)
    ])
    if params.get("minimize_memory_usage", None) is not None:
        cargs.extend([
            "-u",
            ("1" if params.get("minimize_memory_usage", None) else "0")
        ])
    if params.get("winsorize_outliers", None) is not None:
        cargs.extend([
            "-w",
            params.get("winsorize_outliers", None)
        ])
    if params.get("use_euclidean_distance", None) is not None:
        cargs.extend([
            "-e",
            ("1" if params.get("use_euclidean_distance", None) else "0")
        ])
    if params.get("label_propagation", None) is not None:
        cargs.extend([
            "-l",
            params.get("label_propagation", None)
        ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "-v",
            ("1" if params.get("verbose", None) else "0")
        ])
    return cargs


def atropos_outputs(
    params: AtroposParamsDict,
    execution: Execution,
) -> AtroposOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AtroposOutputs(
        root=execution.output_file("."),
        classified_image=execution.output_file(params.get("output", None) + "_classified.nii.gz"),
        posterior_probability_images=execution.output_file("[POSTERIOR_PROBABILITY_IMAGE_FILE_NAME_FORMAT]"),
    )
    return ret


def atropos_execute(
    params: AtroposParamsDict,
    runner: Runner | None = None,
) -> AtroposOutputs:
    """
    Atropos
    
    Atropos is a finite mixture modeling (FMM) segmentation approach that allows
    for prior constraints including a prior label image, prior probability
    images, and/or an MRF prior to enforce spatial smoothing of the labels.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AtroposOutputs`).
    """
    atropos_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ATROPOS_METADATA)
    params = execution.params(params)
    cargs = atropos_cargs(params, execution)
    ret = atropos_outputs(params, execution)
    execution.run(cargs)
    return ret


def atropos(
    intensity_image: str,
    initialization: str,
    mask_image: InputPathType,
    convergence: str,
    likelihood_model: str,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    bspline: str | None = None,
    partial_volume_label_set: str | None = None,
    use_partial_volume_likelihoods: bool | None = None,
    posterior_formulation: str | None = None,
    mrf: str | None = None,
    icm: str | None = None,
    use_random_seed: bool | None = None,
    minimize_memory_usage: bool | None = None,
    winsorize_outliers: str | None = None,
    use_euclidean_distance: bool | None = None,
    label_propagation: str | None = None,
    verbose: bool | None = None,
    runner: Runner | None = None,
) -> AtroposOutputs:
    """
    Atropos
    
    Atropos is a finite mixture modeling (FMM) segmentation approach that allows
    for prior constraints including a prior label image, prior probability
    images, and/or an MRF prior to enforce spatial smoothing of the labels.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        intensity_image: One or more scalar images is specified for\
            segmentation. For scenarios with no prior information, the first scalar\
            image is used to order labelings by intensity. The optional adaptive\
            smoothing weight is applicable with prior images, specified between\
            [0,1].
        initialization: Initialize the FMM parameters. options include Random,\
            Otsu, KMeans, PriorProbabilityImages, and PriorLabelImage.
        mask_image: The required image mask defines the region to be labeled by\
            Atropos.
        convergence: Determine convergence based on mean maximum posterior\
            probability over region of interest.
        likelihood_model: Specify parametric or non-parametric likelihood\
            model. Options include Gaussian, HistogramParzenWindows,\
            ManifoldParzenWindows, among others.
        output: Output labeled image and optionally posterior probability\
            images.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, Atropos tries to infer\
            the dimensionality from the first input image.
        bspline: Parameters for B-Spline. Adaptive smoothing is applied to\
            intensity images if smoothing weights > 0.
        partial_volume_label_set: Model mixtures of classes within single\
            voxels. Specify labels for each partial volume class.
        use_partial_volume_likelihoods: Whether to use partial volume\
            likelihoods. A value of 1 considers the partial volume class separate\
            from tissue classes.
        posterior_formulation: Specify posterior probability formulation.\
            Options are Socrates, Plato, Aristotle, or Sigmoid.
        mrf: Markov Random Field parameters to enforce spatial constraints on\
            segmentation.
        icm: ICM (Iterated Conditional Modes) parameters for asynchronous\
            updating.
        use_random_seed: Initialize with a random seed or a constant seed\
            number.
        minimize_memory_usage: Minimize memory usage by calculating images on\
            the fly and storing only non-negligible pixel values.
        winsorize_outliers: Options to remove effects of outliers in\
            calculations using methods like BoxPlot or GrubbsRosner.
        use_euclidean_distance: Propagate labels throughout the mask using a\
            distance transform.
        label_propagation: Control propagation of each prior label by specified\
            lambda and boundary probability.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AtroposOutputs`).
    """
    params = atropos_params(
        image_dimensionality=image_dimensionality,
        intensity_image=intensity_image,
        bspline=bspline,
        initialization=initialization,
        partial_volume_label_set=partial_volume_label_set,
        use_partial_volume_likelihoods=use_partial_volume_likelihoods,
        posterior_formulation=posterior_formulation,
        mask_image=mask_image,
        convergence=convergence,
        likelihood_model=likelihood_model,
        mrf=mrf,
        icm=icm,
        use_random_seed=use_random_seed,
        output=output,
        minimize_memory_usage=minimize_memory_usage,
        winsorize_outliers=winsorize_outliers,
        use_euclidean_distance=use_euclidean_distance,
        label_propagation=label_propagation,
        verbose=verbose,
    )
    return atropos_execute(params, runner)


__all__ = [
    "ATROPOS_METADATA",
    "AtroposOutputs",
    "AtroposParamsDict",
    "AtroposParamsDictTagged",
    "atropos",
    "atropos_execute",
    "atropos_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_GEOMETRY_MEASURES_METADATA = Metadata(
    id="e086bc064897b80e13ee956cedd1472309a2db39.boutiques",
    name="LabelGeometryMeasures",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_LabelGeometryMeasuresParamsDictNoTag = typing.TypedDict('_LabelGeometryMeasuresParamsDictNoTag', {
    "image_dimension": int,
    "label_image": InputPathType,
    "intensity_image": typing.NotRequired[str | None],
    "csv_file": typing.NotRequired[InputPathType | None],
})
LabelGeometryMeasuresParamsDictTagged = typing.TypedDict('LabelGeometryMeasuresParamsDictTagged', {
    "@type": typing.Literal["ants/LabelGeometryMeasures"],
    "image_dimension": int,
    "label_image": InputPathType,
    "intensity_image": typing.NotRequired[str | None],
    "csv_file": typing.NotRequired[InputPathType | None],
})
LabelGeometryMeasuresParamsDict = _LabelGeometryMeasuresParamsDictNoTag | LabelGeometryMeasuresParamsDictTagged


class LabelGeometryMeasuresOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelGeometryMeasuresParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_csv: OutputPathType | None
    """The CSV file containing the geometry measures."""


def label_geometry_measures_params(
    image_dimension: int,
    label_image: InputPathType,
    intensity_image: str | None = None,
    csv_file: InputPathType | None = None,
) -> LabelGeometryMeasuresParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the input images (e.g., 2 for\
            2D, 3 for 3D).
        label_image: The label image on which geometry measures are computed.
        intensity_image: An optional intensity image for computing\
            intensity-weighted measures. Use "none" or "na" if not provided.
        csv_file: The output file where the geometry measures are written in\
            CSV format.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/LabelGeometryMeasures",
        "image_dimension": image_dimension,
        "label_image": label_image,
    }
    if intensity_image is not None:
        params["intensity_image"] = intensity_image
    if csv_file is not None:
        params["csv_file"] = csv_file
    return params


def label_geometry_measures_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelGeometryMeasuresParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("label_image", None) is None:
        raise StyxValidationError("`label_image` must not be None")
    if not isinstance(params["label_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label_image` has the wrong type: Received `{type(params.get("label_image", None))}` expected `InputPathType`')
    if params.get("intensity_image", None) is not None:
        if not isinstance(params["intensity_image"], str):
            raise StyxValidationError(f'`intensity_image` has the wrong type: Received `{type(params.get("intensity_image", None))}` expected `str | None`')
    if params.get("csv_file", None) is not None:
        if not isinstance(params["csv_file"], (pathlib.Path, str)):
            raise StyxValidationError(f'`csv_file` has the wrong type: Received `{type(params.get("csv_file", None))}` expected `InputPathType | None`')


def label_geometry_measures_cargs(
    params: LabelGeometryMeasuresParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("LabelGeometryMeasures")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("label_image", None)))
    if params.get("intensity_image", None) is not None:
        cargs.append(params.get("intensity_image", None))
    if params.get("csv_file", None) is not None:
        cargs.append(execution.input_file(params.get("csv_file", None)))
    return cargs


def label_geometry_measures_outputs(
    params: LabelGeometryMeasuresParamsDict,
    execution: Execution,
) -> LabelGeometryMeasuresOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelGeometryMeasuresOutputs(
        root=execution.output_file("."),
        output_csv=execution.output_file(pathlib.Path(params.get("csv_file", None)).name) if (params.get("csv_file") is not None) else None,
    )
    return ret


def label_geometry_measures_execute(
    params: LabelGeometryMeasuresParamsDict,
    runner: Runner | None = None,
) -> LabelGeometryMeasuresOutputs:
    """
    LabelGeometryMeasures
    
    This tool computes various geometry measures on a label image, optionally
    using an intensity image, and outputs the results to a CSV file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelGeometryMeasuresOutputs`).
    """
    label_geometry_measures_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_GEOMETRY_MEASURES_METADATA)
    params = execution.params(params)
    cargs = label_geometry_measures_cargs(params, execution)
    ret = label_geometry_measures_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_geometry_measures(
    image_dimension: int,
    label_image: InputPathType,
    intensity_image: str | None = None,
    csv_file: InputPathType | None = None,
    runner: Runner | None = None,
) -> LabelGeometryMeasuresOutputs:
    """
    LabelGeometryMeasures
    
    This tool computes various geometry measures on a label image, optionally
    using an intensity image, and outputs the results to a CSV file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the input images (e.g., 2 for\
            2D, 3 for 3D).
        label_image: The label image on which geometry measures are computed.
        intensity_image: An optional intensity image for computing\
            intensity-weighted measures. Use "none" or "na" if not provided.
        csv_file: The output file where the geometry measures are written in\
            CSV format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelGeometryMeasuresOutputs`).
    """
    params = label_geometry_measures_params(
        image_dimension=image_dimension,
        label_image=label_image,
        intensity_image=intensity_image,
        csv_file=csv_file,
    )
    return label_geometry_measures_execute(params, runner)


__all__ = [
    "LABEL_GEOMETRY_MEASURES_METADATA",
    "LabelGeometryMeasuresOutputs",
    "LabelGeometryMeasuresParamsDict",
    "LabelGeometryMeasuresParamsDictTagged",
    "label_geometry_measures",
    "label_geometry_measures_execute",
    "label_geometry_measures_params",
]
